// EmClient.cpp
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "OpCode.h"
#include "TaskProcessorSockets.h"
#include "../EmEngine.h"
#include "../../OtherFunctions.h"

CEmClient::~CEmClient()
{
	if (closesocket(m_hSocket))
		AddDebugLogLine(false, "closesocket failed");
}

T_CLIENT_TYPE CEmClient_Peer::GetType() const
{
	return T_CLIENT_PEER;
}
T_CLIENT_TYPE CEmClient_Server::GetType() const
{
	return T_CLIENT_SERVER;
}
T_CLIENT_TYPE CEmClient_Web::GetType() const
{
	return T_CLIENT_WEB;
}

void CEmClient::OnConnected(CEmEngine&)
{
}
void CEmClient::OnAccepted(CEmEngine&)
{
}
void CEmClient::OnDisconnected(CEmEngine&)
{
}

//////////////////////////////////////////////////////////////////////
// client connected
// (!) can be both we connected to him or he connected to us
// sending HELLO opcode
void CEmClient_Peer::OnConnected(CEmEngine& stEngine)
{
	if(!m_bFromOutside)
	{
		COpCode_HELLO stMsg;
		md4cpy(stMsg._Hash, stEngine.GetUserHash());
		stMsg._Client._Addr = stEngine.GetClientID();
		stMsg._Client._Port = stEngine.GetPort();
		stMsg._UserName.m_strValue = stEngine.GetUserNick();
		stMsg._UserName.m_bValid = TRUE;
		stMsg._ClientVersion.m_dwValue = EDONKEYVERSION;
		stMsg._ClientVersion.m_bValid = TRUE;
		stMsg._UserPort.m_dwValue = stEngine.GetPort();
		stMsg._UserPort.m_bValid = TRUE;
		if(stEngine.IsConnected())
		{
			stMsg._Server._Addr = stEngine.GetConnectedState()->nAddr;
			stMsg._Server._Port = stEngine.GetConnectedState()->nPort;
		}
		else
		{
			stMsg._Server._Addr = 0;
			stMsg._Server._Port = 0;
		}

		stEngine.SendOpCode(m_hSocket, stMsg, this);
	}
}

//////////////////////////////////////////////////////////////////////
// connected to server
// disconnecting from previous connected server (if any)
// sending LOGINREQUEST opcode
void CEmClient_Server::OnConnected(CEmEngine& stEngine)
{
	// Reset message count
	m_nMsgCount = 0;

	// Disconnect from previous connected server, if any
	if(stEngine.GetServerSocket() != m_hSocket &&
		stEngine.GetServerSocket())
		stEngine.DisconnectFromServer(stEngine.GetServerSocket());

	// Set connection info
	stEngine.ConnectedTo(this);
	stEngine.SetConnectionState(SERVER_WAITFORLOGIN);

	COpCode_LOGINREQUEST stMsg;
	md4cpy(stMsg._Hash, stEngine.GetUserHash());
	stMsg._ClientID = stEngine.GetClientID();
	stMsg._ClientPort = stEngine.GetPort();
	stMsg._UserName.m_strValue = stEngine.GetUserNick();
	stMsg._UserName.m_bValid = true;
	stMsg._ClientVersion.m_dwValue = EDONKEYVERSION;
	stMsg._ClientVersion.m_bValid = true;
	stMsg._UserPort.m_dwValue = stEngine.GetPort();
	stMsg._UserPort.m_bValid = true;

	stEngine.SendOpCode(m_hSocket, stMsg, this);
}

//////////////////////////////////////////////////////////////////////
// disconnected from server
// (!) at any reason, can be even if we tried to connect and failed
void CEmClient_Server::OnDisconnected(CEmEngine& stEngine)
{
	// Set connection info
	if(stEngine.GetServerSocket() == m_hSocket)
		stEngine.SetConnectionState(SERVER_DISCONNECTED);
	else
		// removed if, because we can get 'disconnected' even if we didn't
		// succeeded to connect yet, thus GetConnectingServer() won't return
		// correct value
		// if(stEngine.GetConnectingServer() == m_hSocket)
		stEngine.SetConnectionState(SERVER_DISCONNECTED);
		// TODO: mark somewhere that server is dead
}

//////////////////////////////////////////////////////////////////////
// in the future - move webserver to this engine
void CEmClient_Web::OnConnected(CEmEngine& stEngine)
{
	// say something...
	char szSend[] = "GET / HTTP 1.1\r\n\r\n";
	stEngine.m_stTcp.AllocSend(m_hSocket, szSend, sizeof(szSend) - 1);
}
