// XML.cpp: implementation of the CXmlTask class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "../EmEngine.h"
#include "XML.h"

CXmlTask::CXmlTask()
{
	m_ulId = 0;
}

CString CXmlTask::PrepareResult()
{
	bool bOk = true;
	CMuleXml xml;
	bOk &= xml.Create();
	bOk &= xml.CreateElement(_T("result"));
	bOk &= xml.SetAttribute(_T("id"), m_ulId);
	CString sXml = xml.GetXml();
	if(!bOk || sXml.IsEmpty())
		throw CString(_T("Can't create XML"));
	TRACE("RETURN: %s (XML)\n", sXml);
	return sXml;
}

// Removing old definitions
#undef BEGIN_XML_CMD
#undef XML_PARAM_STRING
#undef XML_PARAM_ULONG
#undef END_XML_CMD

// Defining LoadParams function
#define BEGIN_XML_CMD(name)		bool CXml_##name::LoadParams(MSXML::IXMLDOMNodePtr spNode) { m_spNode = spNode;
#define XML_PARAM_STRING(var)		m_str_##var = GetAttribute(_T(#var));
#define XML_PARAM_ULONG(var)		m_ul_##var = _ttol(GetAttribute(_T(#var)));
#define END_XML_CMD				return true; }
#include "XMLMsgs.h"

//////////////////////////////////////////////////////////////////////
// Parse XML command
// Each call to this function should be in separate thread,
// cause it won't return until command execution is finished.
// E.g. if you command to connect to server, it will return
// immediately - cause connection task is asynchronous,
// but if you will demand file or client list, it will wait until
// list is populated and only then return.
void CEmEngine::ParseXmlCommand(CString sXml)
{
	try
	{
		// Load xml
		CMuleXml xml;
		if(!xml.Load(sXml))
			throw CString(_T("Bad XML"));

		// Get basic parameters - <cmd name="xxx" id="nnn" ... />
		if(!xml.SelectNode(_T("./cmd")))
			throw CString(_T("Can't find <cmd> tag"));
		CString sCmd = xml.GetAttribute(_T("name"));
		if(sCmd.IsEmpty())
			throw CString(_T("Error reading \"name\" parameter of <cmd> tag"));
		ULONG ulId = _ttol(xml.GetAttribute(_T("id")));
		if(!ulId)
			throw CString(_T("Error reading \"id\" parameter of <cmd> tag"));

		CXmlTask *pTask = NULL;

		// Removing old definitions
		#undef BEGIN_XML_CMD
		#undef XML_PARAM_STRING
		#undef XML_PARAM_ULONG
		#undef END_XML_CMD

		// Switch by command name, creating appropriate task and load its parameters
		#define BEGIN_XML_CMD(name)		if(!sCmd.CompareNoCase(_T(#name))) pTask = new CXml_##name;
		#define XML_PARAM_STRING(var)
		#define XML_PARAM_ULONG(var)
		#define END_XML_CMD
		#include "XMLMsgs.h"

		if(pTask)
		{
			pTask->m_ulId = ulId;
			if(!pTask->LoadParams(xml.GetNode())) 
				delete pTask;
			else
				m_stMainTP.Push(pTask);
		}
	}
	catch(CString &obj)
	{
		// should return <result id="0" error="..." />
		TRACE("ERROR: (XML) %s\n", obj);
		ASSERT(FALSE);
	}
	catch(...)
	{
		// should return <result id="0" error="bad_xml errorcode" />
		TRACE("ERROR: (XML) bad xml\n");
		ASSERT(FALSE);
	}
}

XML_PROCESS_FUNC(connect)
{
	stEngine.ConnectToServer(m_str_addr, m_ul_port);

	PrepareResult();

	return true;
}