// parts of this file are based on work from pan One (http://home-3.tiscali.nl/~meost/pms/)

//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "knownfile.h"
#include "opcodes.h"
#include <io.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "emule.h"
#include "ini2.h"
#include "QArray.h"	//netwolf: complete sources (zegzav) 13.05.03
#include "otherfunctions.h"

#if JUMPSTART
#include "Jumpstarter.h"
#endif

	
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CAbstractFile::CreateED2kLink() const
{
	CString		strLink;

	strLink.Format( _T("ed2k://|file|%s|%u|%s|/"),
					g_eMuleApp.StripInvalidFilenameChars(GetFileName(), false),	// spaces to dots
					GetFileSize(),
					EncodeBase16(GetFileHash(),16) );

	return strLink;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CAbstractFile::CreateED2kSourceLink() const
{
	uint32		dwID = 0;

#ifdef OLD_SOCKETS_ENABLED
	if (!g_eMuleApp.m_pServerConnect->IsConnected() || g_eMuleApp.m_pServerConnect->IsLowID())
	{
		return CString("");
	}
	dwID = g_eMuleApp.m_pServerConnect->GetClientID();
#endif //OLD_SOCKETS_ENABLED

	CString		strLink;

	strLink.Format( _T("ed2k://|file|%s|%u|%s|/|sources,%i.%i.%i.%i:%i|/"),
					g_eMuleApp.StripInvalidFilenameChars(GetFileName(), false),	// spaces to dots
					GetFileSize(),
					EncodeBase16(GetFileHash(),16),
					static_cast<BYTE>(dwID),
					static_cast<BYTE>(dwID>>8),
					static_cast<BYTE>(dwID>>16),
					static_cast<BYTE>(dwID>>24),
					g_eMuleApp.m_pGlobPrefs->GetListenPort() );

	return strLink;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CAbstractFile::CreateHTMLED2kLink() const
{
	return CString(_T("<a href=\"") + CreateED2kLink() + _T("\">") + g_eMuleApp.StripInvalidFilenameChars(GetFileName(), true) + _T("</a>"));
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CFileStatistic::CFileStatistic()
{
//--- xrmb:parttraffic ---
	EMULE_TRY

	partTraffic = NULL;
	partTrafficSession = NULL;

	blockTraffic = NULL;
	blockTrafficSession = NULL;

	partAccepted = NULL;
	partAcceptedSession = NULL;

	completeReleases = 0.0;
	m_iNumRequested = m_qwNumTransferred = m_iNumAccepted = alltimerequested= alltimetransferred = alltimeaccepted = 0;
	fileParent	=	NULL;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CFileStatistic::~CFileStatistic()	
{
	EMULE_TRY

	if (partTraffic)			delete[] partTraffic;			partTraffic=NULL;
	if (partTrafficSession)		delete[] partTrafficSession;	partTrafficSession=NULL;

	if (partAccepted)			delete[] partAccepted;			partAccepted=NULL;
	if (partAcceptedSession)	delete[] partAcceptedSession;	partAcceptedSession=NULL;

	if (blockTraffic)			delete[] blockTraffic;			blockTraffic=NULL;
	if (blockTrafficSession)	delete[] blockTrafficSession;	blockTrafficSession=NULL;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFileStatistic::AddRequest()
{
	EMULE_TRY

	m_iNumRequested++;
	alltimerequested++;
	g_eMuleApp.m_pKnownFilesList->m_iNumRequested++;
	if (::IsWindow(g_eMuleApp.m_pdlgEmule->m_wndSharedFiles.m_hWnd))	//fix crash at exit [TwoBottle Mod]
	{
		fileParent->UpdateSharedFileDisplay();
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFileStatistic::AddAccepted()
{
	EMULE_TRY

	m_iNumAccepted++;
	alltimeaccepted++;
	g_eMuleApp.m_pKnownFilesList->m_iNumAccepted++;
	fileParent->UpdateSharedFileDisplay();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CFileStatistic::AddTransferred(uint32 bytes)
{
	EMULE_TRY

	m_qwNumTransferred += bytes;
	alltimetransferred += bytes;
	g_eMuleApp.m_pKnownFilesList->m_iNumTransferred += bytes;
	fileParent->UpdateSharedFileDisplay();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CKnownFile::CKnownFile()
{
	m_lFileSize = 0;
	m_timetLastWriteDate = 0;
	m_iPriority = PR_NORMAL;

	if (g_eMuleApp.m_pGlobPrefs->IsUAPEnabled())
		m_bAutoPriority = true;
	else
		m_bAutoPriority = false;

	m_iPermissions = PERM_ALL;
	statistic.fileParent=this;
	m_bCommentLoaded=false;
	m_iMoviePreviewMode=0;
	m_eRating = PF_RATING_NONE;
	m_strComment="";
	// m_iPartCount = 0;	//netwolf // commented out for Complete sources
	m_nCompleteSourcesCount= 0;		//  netwolf: complete sources (zegzav) 13.05.03
	m_nCompleteSourcesCountLo= 0;
	m_nCompleteSourcesCountHi= 0;
	m_nCompleteSourcesTime= 0;
	
	m_partHashArray.RemoveAll();
	m_tagArray.RemoveAll();

	//--- xrmb:allyaknow ---
	m_sharedFile=false;
	//--- :xrmb ---
	md4clr(m_fileHash);

#if JUMPSTART
    m_Jumpstarter = NULL;
#endif
	m_bOfferedToServer= true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CKnownFile::~CKnownFile()
{
	EMULE_TRY
	CSingleLock Lock(&m_mutexHashList,TRUE);
	for (int i = 0; i < m_partHashArray.GetSize(); i++)
		if (m_partHashArray[i])
			delete[] m_partHashArray[i];
	Lock.Unlock();
	for (int i = 0; i < m_tagArray.GetSize(); i++)
		safe_delete(m_tagArray[i]);
	
#if JUMPSTART
	if (m_Jumpstarter)
		delete m_Jumpstarter;
#endif

    EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CKnownFile::CalculateCompleteSources()
{
	EMULE_TRY

	if ((!this->IsPartFile()) && ((::GetTickCount() - m_nCompleteSourcesTime) >= 2*60*1000))	// 2 min.
	{
		CQArray<uint16,uint16> count;
		CTypedPtrList<CPtrList, CUpDownClient*> clientuploadlist;
		m_nCompleteSourcesCount= m_nCompleteSourcesCountLo= m_nCompleteSourcesCountHi= 0;
		g_eMuleApp.m_pClientList->GetClientListByFileID(&clientuploadlist, GetFileHash());
		count.SetSize(0, clientuploadlist.GetSize());
		for (POSITION pos= clientuploadlist.GetHeadPosition(); pos != 0; )
		{
			CUpDownClient *cur_src= clientuploadlist.GetNext(pos);
			uint16 cur_count= cur_src->GetUpCompleteSourcesCount();
			if ((cur_count) && ((::GetTickCount() - cur_src->GetUpCompleteSourcesTime()) < 30*60*1000))	// 30 min.
				count.Add(cur_count);
		}
		count.FreeExtra();

		int32 n= count.GetSize();
		if (n > 0)
		{
			count.QuickSort();

			// calculate range
			int32 i= n >> 1;		// (n / 2)
			int32 j= (n * 3) >> 2;	// (n * 3) / 4
			int32 k= (n * 7) >> 3;	// (n * 7) / 8
			if (n < 5)
			{
				m_nCompleteSourcesCount= count.GetAt(i);
				m_nCompleteSourcesCountLo= 0;
				m_nCompleteSourcesCountHi= m_nCompleteSourcesCount;
			}
			else if (n < 10)
			{
				m_nCompleteSourcesCount= count.GetAt(i);
				m_nCompleteSourcesCountLo= count.GetAt(i - 1);
				m_nCompleteSourcesCountHi= count.GetAt(i + 1);
			}
			else if (n < 20)
			{
				m_nCompleteSourcesCount= count.GetAt(i);
				m_nCompleteSourcesCountLo= count.GetAt(i);
				m_nCompleteSourcesCountHi= count.GetAt(j);
			}
			else
			{
				m_nCompleteSourcesCount= count.GetAt(j);
				m_nCompleteSourcesCountLo= m_nCompleteSourcesCount;
				m_nCompleteSourcesCountHi= count.GetAt(k);
			}
		}

		m_nCompleteSourcesTime= ::GetTickCount();
	}

	UpdateUploadAutoPriority();
	UpdateSharedFileDisplay();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CKnownFile::UpdateSharedFileDisplay()
{
	if (g_eMuleApp.m_pdlgEmule->m_wndSharedFiles.m_ctlSharedFilesList.m_hWnd != NULL)
	{
		g_eMuleApp.m_pdlgEmule->m_wndSharedFiles.m_ctlSharedFilesList.PostMessage( WM_SFL_UPDATEITEM,
																				static_cast<WPARAM>(g_eMuleApp.m_pdlgEmule->m_wndSharedFiles.IsWindowVisible()),
																				reinterpret_cast<LPARAM>(dynamic_cast<CKnownFile*>(this)) );
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CKnownFile::SetFilePath(LPCTSTR pszFilePath)
{
	m_strFilePath = pszFilePath;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CKnownFile::CreateFromFile(const CString &strDirectory,const CString &strFileName)
{
	EMULE_TRY

//	TODO: Errorhandling

//
//	First create the filehashset
//

//
//	Try to open the specified file
	directory = strDirectory;

	CString			strNameBuffer;

	strNameBuffer.Format(_T("%s\\%s"),strDirectory,strFileName);
	SetFilePath(strNameBuffer);

	FILE		*file = _tfopen(strNameBuffer,_T("rbS"));	

//	If we failed to open the spec'd file, return false
	if (!file)
		return false;

	m_strFileName = strFileName;
	m_lFileSize =_filelength(file->_file);

	AddDebugLogLine(false, RGB_LOG_DIMMED + _T("File %s is being hashed"), m_strFileName);

//	Create hashset
	uint32		dwBytesLeft = m_lFileSize;

	for (uint16 dwHashCount = 0; dwBytesLeft >= PARTSIZE;)
	{
		uchar		*pNewHash = new uchar[16];

		md4clr(pNewHash);	//SyruS needed or not...?

	//	Create a hash for the next PARTSIZE bytes of 'file'
		CreateHashFromFile(file,PARTSIZE,pNewHash);
	//	SyruS SLUGFILLER: SafeHash - quick fallback
		if (!g_eMuleApp.m_pdlgEmule->IsRunning())	// in case of shutdown while still hashing
		{
			fclose(file);
			delete[] pNewHash;
			return false;
		}

		CSingleLock Lock(&m_mutexHashList,TRUE);
		m_partHashArray.Add(pNewHash);
		dwBytesLeft -= PARTSIZE;
		dwHashCount++;
	}
	uchar* lasthash = new uchar[16];
	md4clr(lasthash);	//SyruS ...why needed here?
	CreateHashFromFile(file,dwBytesLeft,lasthash);
	if (!dwHashCount)
	{
		md4cpy(m_fileHash,lasthash);
        	delete[] lasthash; // i_a: memleak
	}
	else 
	{
		CSingleLock Lock(&m_mutexHashList,TRUE);
		m_partHashArray.Add(lasthash);
		uchar* buffer = new uchar[m_partHashArray.GetCount()*16];
		for (int i = 0;i < m_partHashArray.GetCount();i++)
			md4cpy(buffer+(i*16),m_partHashArray[i]);
		CreateHashFromString(buffer,m_partHashArray.GetCount()*16,m_fileHash);
		Lock.Unlock();
		delete[] buffer;
	}
	// TODO: Add filetags

	// set lastwrite date
	struct _stat fileinfo;
	_fstat(file->_file,&fileinfo);
	m_timetLastWriteDate = fileinfo.st_mtime;
	//finished
	fclose(file);
	return true;	

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadHashsetFromFile() loads the KnownFile with a hashset from the file 'file' in the format of "known.met".
//		If 'bCheckHash' is true, the file hash in 'file' must match a file hash calculated from the part hashes
//		'file'. false is returned on failure, true on success.
bool CKnownFile::LoadHashsetFromFile(CFile &file, bool bCheckHash)
{
//	Needed for memfiles. It's probably better to switch everything to CFile...
	EMULE_TRY

//	Read the file hash
	uchar		checkid[16];

	file.Read(&checkid,16);			// <HASH> file hash

//	Read the number of parts
	uint16		iNumParts;

	file.Read(&iNumParts,2);		// <count:WORD> number of parts

	INT_PTR		iParts = static_cast<int16>(iNumParts);

	if (iParts < 0)		//Cax2 - why? MOREVIT - Yes, really. Why?? Not used elsewhere.
	{
		if (!m_strFileName.IsEmpty() )
		{
		//	TODO: Add some way to tell if the file is really "known.met" or a memfile.
			AddDebugLogLine(false, _T("Loaded hashset size is corrupt in known.met, file: %s!"), m_strFileName);
		}
		else
		{
			AddDebugLogLine(false, _T("Loaded hashset size is corrupt in known.met !"));		
		}
	}
	CSingleLock		Lock(&m_mutexHashList,TRUE);

//
//	Read the part hashes.
	for (int i = 0; i < iNumParts; i++)
	{
		uchar		*pHash = new uchar[16];

		file.Read(pHash,16);		// <HASH>[count] part hashes
		m_partHashArray.Add(pHash);
	}
//	SLUGFILLER: hashlistRecheck - always check m_partHashArray
	if (!bCheckHash)
	{
		md4cpy(m_fileHash,checkid);
		if (!iNumParts) // nothing to check
			return true;
	}
	else
	{
		if (!iNumParts) // data not sent
			return false;
	//	If the number of hashes doesn't match the number of parts in the file...
		if (iNumParts != GetPartCount())
		{
		//	Delete the part hash array
			for (int i = 0; i < m_partHashArray.GetSize(); i++)
				delete[] m_partHashArray[i];
			m_partHashArray.RemoveAll();
			return false;
		}
	}
// SLUGFILLER: hashlistRecheck
// trust noone;-)

	if (!m_partHashArray.IsEmpty())
	{
	//	Glom all the part hashes into one big array and hash the hashes
		uchar		*buffer = new uchar[m_partHashArray.GetCount()*16];

		for (int i = 0; i < m_partHashArray.GetCount(); i++)
			md4cpy(buffer+(i*16),m_partHashArray[i]);
		CreateHashFromString(buffer,m_partHashArray.GetCount()*16,checkid);
		delete[] buffer;
	}
//	If the file hash from 'file' matches the calculated hash...
	if (md4cmp(m_fileHash,checkid) == 0)
		return true;
	else
	{
	//	Delete the part hash array
		for (int i = 0; i < m_partHashArray.GetSize(); i++)
		{	
			delete[] m_partHashArray[i];
			m_partHashArray[i]= NULL;	// most likely this is needless, but on the other hand...
		}
		m_partHashArray.RemoveAll();
		return false;
	}

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadTagsFromFile() parses what tags we know and stores the rest in 'm_tagArray' so they can be rewritten.
bool CKnownFile::LoadTagsFromFile(CFile &file)
{
	EMULE_TRY

	uint32		dwNumTags;

	file.Read(&dwNumTags,4);							// {<TAGCOUNT:DWORD>

	for (uint32 j = 0; j != dwNumTags; j++)
	{
		CFileTag		*newtag = new CFileTag(file);

		switch (newtag->GetSpecialTag())
		{
			case FT_FILENAME:							// (FT_FILENAME:string) file name
			{
				//m_strFileName = newtag->GetStringValue();
				SetFileName(newtag->GetStringValue());
				delete newtag;
				break;
			}
			case FT_FILESIZE:							// (FT_FILESIZE:int) file size (bytes)
			{
				m_lFileSize = newtag->GetIntValue();
				delete newtag;
				break;
			}
			case FT_ATTRANSFERRED:						// (FT_ATTRANSFERRED:int) all time transferred (low long)
			{
				statistic.alltimetransferred = newtag->GetIntValue();
				delete newtag;
				break;
			}
			case FT_ATTRANSFERREDHI:					// (FT_ATTRANSFERREDHI:int) all time transferred (high long)
			{
				uint32		dwHigh,dwLow;

				dwLow = statistic.alltimetransferred;
				dwHigh = newtag->GetIntValue();

				uint64		qwHigh;

				qwHigh = dwHigh;
				qwHigh <<= 32;
				statistic.alltimetransferred = dwLow+qwHigh;
				delete newtag;
				break;
			}
			case FT_ATREQUESTED:						// (FT_ATREQUESTED:int) all time requested
			{
				statistic.alltimerequested = newtag->GetIntValue();
				delete newtag;
				break;
			}
 			case FT_ATACCEPTED:							// (FT_ATACCEPTED:int) all time accepted
			{
				statistic.alltimeaccepted = newtag->GetIntValue();
				delete newtag;
				break;
			}
			case FT_PRIORITY:							// (FT_PRIORITY:int) upload priority
			{
				m_iPriority = newtag->GetIntValue();
				if (m_iPriority == PR_AUTO)
				{
					SetAutoPriority(true);
					UpdateUploadAutoPriority();
				}
				else
				{
					SetAutoPriority(false);
				}
				delete newtag;
				break;
			}
			case FT_PERMISSIONS:						// (FT_PERMISSIONS:int) upload permissions
			{
				m_iPermissions = newtag->GetIntValue();
				delete newtag;
				break;
			}
			default:
				m_tagArray.Add(newtag);
		}	
	}
	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CKnownFile::LoadDateFromFile(CFile& file)
{
	EMULE_TRY

	file.Read(&m_timetLastWriteDate,4);
	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CKnownFile::LoadFromFile(CFile& file)
{
	EMULE_TRY

    bool		result;
    bool		hashset_ok = true;

	result = LoadDateFromFile(file);				// <DATE:time_t> last write date/time
    hashset_ok = LoadHashsetFromFile(file,false);	// (HASHSET)
    result = result && LoadTagsFromFile(file);		// <Tag_set>
    result = result && hashset_ok;

//	Cax2 - if m_strFileName==NULL, don't report it, it should still be ok...
	if(!hashset_ok && !m_strFileName.IsEmpty())
        AddLogLine(false, RGB_LOG_WARNING + GetResString(IDS_ERR_KNOWNHASHCORR), m_strFileName);

	// SLUGFILLER: hashlistRecheck - read all data before error-checking
	if(result)
		if (GetPartCount() != 1 && GetPartCount() != m_partHashArray.GetCount())
			result = false;
	// SLUGFILLER: hashlistRecheck
#if JUMPSTART
    if (result)
	{
		if(CJumpstarter::ShouldBeEnabledForFile(this))
		{
            m_Jumpstarter = new CJumpstarter(this);
            AddLogLine(false, IDS_JS_ENABLED, m_strFileName);
        }
    }
#endif // JUMPSTART

	return result;

    EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CKnownFile::WriteToFile(FILE* file)
{
	EMULE_TRY

	CSingleLock		Lock(&m_mutexHashList,TRUE);

	uint16		dwNumParts = m_partHashArray.GetCount();
//
//	It's safer to do a check first to avoid file corruption
	for (int i = 0; i < dwNumParts; i++)
	{	
		if (m_partHashArray[i] == NULL)
			return false; 
	}

	fwrite(&m_timetLastWriteDate,4,1,file);		// <DATE:time_t> last write date/time
//
//	hashset
	fwrite(&m_fileHash,16,1,file);				// <HASH>

	fwrite(&dwNumParts,2,1,file);				// <count:WORD> number of parts

	for (i = 0; i < dwNumParts; i++)
	{	
		fwrite(m_partHashArray[i],16,1,file);	// <HASH>[count] part hashes
	}

	Lock.Unlock();

//  DonGato: Deprecated
//	//tags
//	uint32 tagcount = m_tagArray.GetCount()+7;

	//tags
	const int iFixedTags = 8;
	uint32 tagcount = iFixedTags;
	// Float meta tags are currently not written. All older eMule versions < 0.28a have 
	// a bug in the meta tag reading+writing code. To achive maximum backward 
	// compatibility for met files with older eMule versions we just don't write float 
	// tags. This is OK, because we (eMule) do not use float tags. The only float tags 
	// we may have to handle is the '# Sent' tag from the Hybrid, which is pretty 
	// useless but may be received from us via the servers.
	// 
	// The code for writing the float tags SHOULD BE ENABLED in SOME MONTHS (after most 
	// people are using the newer eMule versions which do not write broken float tags).	
	for (int j = 0; j < m_tagArray.GetCount(); j++)
	{
		if (m_tagArray[j]->GetType() == TAGTYPE_STRING || m_tagArray[j]->GetType() == TAGTYPE_INT)
			tagcount++;
	}
//
//	Standard tags
	fwrite(&tagcount,4,1,file);							// {<TAGCOUNT:DWORD>

	CFileTag	nameTag(FT_FILENAME,m_strFileName);

	nameTag.WriteToFile(file);							// (FT_FILENAME:string) file name

	CFileTag	sizeTag(FT_FILESIZE,m_lFileSize);

	sizeTag.WriteToFile(file);							// (FT_FILESIZE:int) file size (bytes)

//	Statistic
	uint32		tran;

	tran = statistic.GetAllTimeTransferred();

	CFileTag	attag1(FT_ATTRANSFERRED, tran);

	attag1.WriteToFile(file);
	tran = statistic.alltimetransferred >> 32;

	CFileTag	attag4(FT_ATTRANSFERREDHI, tran);

	attag4.WriteToFile(file);

	CFileTag	attag2(FT_ATREQUESTED,statistic.GetAllTimeRequests());

	attag2.WriteToFile(file);

	CFileTag	attag3(FT_ATACCEPTED,statistic.GetAllTimeAccepts());

	attag3.WriteToFile(file);
//
//	priority & permissions
	if (IsAutoPrioritized())
	{
		CFileTag		prioTag(FT_PRIORITY,static_cast<uint8>(PR_AUTO));

		prioTag.WriteToFile(file);
	}
	else
	{
		CFileTag		prioTag(FT_PRIORITY, m_iPriority);

		prioTag.WriteToFile(file);
	}

	CFileTag	permTag(FT_PERMISSIONS, m_iPermissions);

	permTag.WriteToFile(file);
//
//	Unknown tags
	for (int j = 0; j < m_tagArray.GetCount(); j++)
	{
		if (m_tagArray[j]->GetType() == TAGTYPE_STRING || m_tagArray[j]->GetType() == TAGTYPE_INT)
			m_tagArray[j]->WriteToFile(file);
	}

	return ferror(file);

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CreateHashFromInput() generates a hash from the next 'Length' bytes of one of 'file', 'file2', or 'in_string'
//		(the other two must be NULL (UGLY UGLY UGLY)). The hash is returned in '*Output'.
void CKnownFile::CreateHashFromInput(FILE* file,CFile* file2, int Length, uchar* Output, uchar* in_string) 
{
	EMULE_TRY

	// time critial
	uint32 Hash[4];

	Hash[0] = 0x67452301;
	Hash[1] = 0xEFCDAB89;
	Hash[2] = 0x98BADCFE;
	Hash[3] = 0x10325476;

	CFile* data = 0;

	if (in_string)
		data = new CMemFile(in_string,Length);

	uint32 Required = Length;
	uchar   X[64*128];

	while (Required >= 64)
	{
		uint32 len = Required & ~63; 

		if (len > sizeof(X)) 
			len = sizeof(X);
		if (in_string)
			data->Read(&X,len);
		else if (file)
			fread(&X,len,1,file);
		else if (file2)
			file2->Read(&X,len);
		uint32 i = 0;
		do
		{ 
			MD4Transform(Hash, (uint32*)(X + i)); 
			i += 64;
		} while(i < len);
		Required -= len;
	}
	// bytes to read
	if (Required != 0)
	{
		if (in_string)
			data->Read(&X,Required);
		else if (file)
			fread(&X,Required,1,file);
		else if (file2)
			file2->Read(&X,Required);
	}
	// in byte scale 512 = 64, 448 = 56
	X[Required++] = 0x80;
	if (Required > 56)
	{
		memset2(&X[Required], 0, 64 - Required);
		MD4Transform(Hash, (uint32*)X);
		Required = 0;
	}
	memset2(&X[Required], 0, 56 - Required);
	// add size (convert to bits)
	uint32 Length2[2] = { Length << 3, (uint32)Length >> 29 };
	memcpy2(&X[56], Length2, 8);
	MD4Transform(Hash, (uint32*)X);
	md4cpy(Output, Hash);
	safe_delete(data);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uchar* CKnownFile::GetPartHash(uint16 part)
{
	EMULE_TRY
	// Cax2 - better placing the mutex in PartFileHashFinished()...
	//CSingleLock Lock(&m_mutexHashList,TRUE);
	if (part >= m_partHashArray.GetCount())
		return 0;

	return m_partHashArray[part];

	EMULE_CATCH

	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
double CKnownFile::GetFileRatio()
{
	EMULE_TRY

//	MatzeHH formula I think is better than herbert's one (Tarod)
	if (g_eMuleApp.m_pKnownFilesList->GetTotalRequested()==0)
		return 1.0;
	if (statistic.GetRequests() == 0)
		return 2.0;

	double		dblRatio;

	dblRatio = (static_cast<double>(statistic.GetRequests())) / (static_cast<double>(g_eMuleApp.m_pKnownFilesList->GetTotalRequested()));
	dblRatio = -4.0 * log10(static_cast<double>(g_eMuleApp.m_pSharedFilesList->GetCount())) * dblRatio;
	dblRatio = 2.0 * exp(dblRatio);

	if (dblRatio < 1.0)
	{
		dblRatio = 1.0;
	}
	else if (dblRatio > 2.0)
	{
		dblRatio = 2.0;
	}

	return dblRatio ;

	EMULE_CATCH

	return 1.0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
uint16 CKnownFile::GetPartCount()
{
	EMULE_TRY

	return (m_lFileSize/PARTSIZE) + ((m_lFileSize % PARTSIZE)? 1:0);

	EMULE_CATCH

	return 0;
}

static void MD4Transform(uint32 Hash[4], uint32 x[16])
{
	EMULE_TRY

	uint32 a = Hash[0];
	uint32 b = Hash[1];
	uint32 c = Hash[2];
	uint32 d = Hash[3];

	/* Round 1 */
	MD4_FF(a, b, c, d, x[ 0], S11); // 01
	MD4_FF(d, a, b, c, x[ 1], S12); // 02
	MD4_FF(c, d, a, b, x[ 2], S13); // 03
	MD4_FF(b, c, d, a, x[ 3], S14); // 04
	MD4_FF(a, b, c, d, x[ 4], S11); // 05
	MD4_FF(d, a, b, c, x[ 5], S12); // 06
	MD4_FF(c, d, a, b, x[ 6], S13); // 07
	MD4_FF(b, c, d, a, x[ 7], S14); // 08
	MD4_FF(a, b, c, d, x[ 8], S11); // 09
	MD4_FF(d, a, b, c, x[ 9], S12); // 10
	MD4_FF(c, d, a, b, x[10], S13); // 11
	MD4_FF(b, c, d, a, x[11], S14); // 12
	MD4_FF(a, b, c, d, x[12], S11); // 13
	MD4_FF(d, a, b, c, x[13], S12); // 14
	MD4_FF(c, d, a, b, x[14], S13); // 15
	MD4_FF(b, c, d, a, x[15], S14); // 16

	/* Round 2 */
	MD4_GG(a, b, c, d, x[ 0], S21); // 17
	MD4_GG(d, a, b, c, x[ 4], S22); // 18
	MD4_GG(c, d, a, b, x[ 8], S23); // 19
	MD4_GG(b, c, d, a, x[12], S24); // 20
	MD4_GG(a, b, c, d, x[ 1], S21); // 21
	MD4_GG(d, a, b, c, x[ 5], S22); // 22
	MD4_GG(c, d, a, b, x[ 9], S23); // 23
	MD4_GG(b, c, d, a, x[13], S24); // 24
	MD4_GG(a, b, c, d, x[ 2], S21); // 25
	MD4_GG(d, a, b, c, x[ 6], S22); // 26
	MD4_GG(c, d, a, b, x[10], S23); // 27
	MD4_GG(b, c, d, a, x[14], S24); // 28
	MD4_GG(a, b, c, d, x[ 3], S21); // 29
	MD4_GG(d, a, b, c, x[ 7], S22); // 30
	MD4_GG(c, d, a, b, x[11], S23); // 31
	MD4_GG(b, c, d, a, x[15], S24); // 32

	/* Round 3 */
	MD4_HH(a, b, c, d, x[ 0], S31); // 33
	MD4_HH(d, a, b, c, x[ 8], S32); // 34
	MD4_HH(c, d, a, b, x[ 4], S33); // 35
	MD4_HH(b, c, d, a, x[12], S34); // 36
	MD4_HH(a, b, c, d, x[ 2], S31); // 37
	MD4_HH(d, a, b, c, x[10], S32); // 38
	MD4_HH(c, d, a, b, x[ 6], S33); // 39
	MD4_HH(b, c, d, a, x[14], S34); // 40
	MD4_HH(a, b, c, d, x[ 1], S31); // 41
	MD4_HH(d, a, b, c, x[ 9], S32); // 42
	MD4_HH(c, d, a, b, x[ 5], S33); // 43
	MD4_HH(b, c, d, a, x[13], S34); // 44
	MD4_HH(a, b, c, d, x[ 3], S31); // 45
	MD4_HH(d, a, b, c, x[11], S32); // 46
	MD4_HH(c, d, a, b, x[ 7], S33); // 47
	MD4_HH(b, c, d, a, x[15], S34); // 48

	Hash[0] += a;
	Hash[1] += b;
	Hash[2] += c;
	Hash[3] += d;

	EMULE_CATCH
}

// Updates priority of file if autopriority is activated
void CKnownFile::UpdateUploadAutoPriority(void)
{
	EMULE_TRY

	if (IsAutoPrioritized())
	{
		if (IsPartFile() && g_eMuleApp.m_pDownloadQueue)
		{
			CPartFile* pPartFile = g_eMuleApp.m_pDownloadQueue->GetFileByID(GetFileHash());
		
			if(pPartFile != NULL)
			{
				int iValidSources = pPartFile->GetSourceCount() - pPartFile->GetNotCurrentSourcesCount();

				if (iValidSources < RARE_FILE)
					SetPriority(PR_VERYHIGH);
				else if (iValidSources < 200)
					SetPriority(PR_HIGH);
				else
					SetPriority(PR_NORMAL);
			}
		}
		else
		{
			int iCompleteSources = GetCompleteSourcesCount();

			if (iCompleteSources < 5)
				SetPriority(PR_VERYHIGH);
			else if (iCompleteSources < RARE_FILE)
				SetPriority(PR_HIGH);
			else
				SetPriority(PR_NORMAL);
		}
	}

	EMULE_CATCH
}

// Adde by Tarod [Juanjo]
void CAbstractFile::SetFileName(const CString& NewName) 
{	
	m_strFileName = NewName; 
	      		
	m_strFileName.Replace('/','-');
	m_strFileName.Replace('>','-');
	m_strFileName.Replace('<','-');
	m_strFileName.Replace('*','-');
	m_strFileName.Replace(':','-');
	m_strFileName.Replace('?','-');
	//SyruS: these 3 were missing.
	m_strFileName.Replace('|','-');
	m_strFileName.Replace('\"','-');
	m_strFileName.Replace('\\','-');	
}
//End by Tarod

CString CAbstractFile::GetFileType(bool bLocalized)
{
	EMULE_TRY

	EED2KFileType eFileType = ED2KFT_ANY;

	if (!m_strFileName.IsEmpty())
		eFileType = GetED2KFileTypeID(m_strFileName);

	if (bLocalized)
	{
		switch (eFileType)
		{
			case ED2KFT_AUDIO :
				return GetResString(IDS_SEARCH_AUDIO);
			case ED2KFT_VIDEO :
				return GetResString(IDS_SEARCH_VIDEO);
			case ED2KFT_IMAGE :
				return GetResString(IDS_SEARCH_PICS);
			case ED2KFT_ARCHIVE :
				return GetResString(IDS_SEARCH_ARC);
			case ED2KFT_PROGRAM :
				return GetResString(IDS_SEARCH_PRG);
			case ED2KFT_CDIMAGE :
				return GetResString(IDS_SEARCH_CDIMG);
			case ED2KFT_DOCUMENT :
				return GetResString(IDS_SEARCH_DOC);
			case ED2KFT_ANY :
			default:
				return GetResString(IDS_SEARCH_ANY);
		}
	}
	else
	{
		switch (eFileType)
		{
			case ED2KFT_AUDIO:
				return "Audio";
			case ED2KFT_VIDEO:
				return "Video";
			case ED2KFT_IMAGE:
				return "Image";
			case ED2KFT_ARCHIVE:
			case ED2KFT_CDIMAGE:
			case ED2KFT_PROGRAM:
				return "Pro";
			case ED2KFT_DOCUMENT:
				return "Doc";
			case ED2KFT_ANY:
			default:
				return "";
		}
	}

	EMULE_CATCH

	return "";
}

Packet*	CKnownFile::CreateSrcInfoPacket(CUpDownClient* forClient)
{
	EMULE_TRY

	CTypedPtrList<CPtrList, CUpDownClient*> srclist;
	g_eMuleApp.m_pUploadQueue->FindSourcesForFileById(&srclist, forClient->m_reqFileHash); //should we use "fileHash"?

	if(srclist.IsEmpty())
		return 0;

	CMemFile data;
	uint16 nCount = 0;

	data.Write(forClient->m_reqFileHash, 16);
	data.Write(&nCount,2);

	uint32 lastRequest = forClient->GetLastSrcReqTime();
	//we are only taking 30 random sources since we can't be sure if they have parts we need
	//this is hard coded because its a temp solution until next(?) version
	srand(time(NULL));
	for(int i = 0; i < 30; i++) {
		int victim = ((rand() >> 7) % srclist.GetSize());
		POSITION pos = srclist.FindIndex(victim);
		CUpDownClient *cur_src = srclist.GetAt(pos);
		// LANCAST (moosetea) - We don't share Lan clients sources as they are private ips/userids
		if(!cur_src->IsOnLAN() && !cur_src->HasLowID() && cur_src != forClient)  {
			nCount++;
			uint32 dwID = cur_src->GetUserID();
			uint16 nPort = cur_src->GetUserPort();
			uint32 dwServerIP = cur_src->GetServerIP();
			uint16 nServerPort = cur_src->GetServerPort();
			data.Write(&dwID,4);
			data.Write(&nPort,2);
			data.Write(&dwServerIP,4);
			data.Write(&nServerPort,2);
		}

		srclist.RemoveAt(pos);
		if(srclist.GetSize() == 0)
			break;
	}
	if (!nCount)
		return 0;
	data.Seek(16,0);
	data.Write(&nCount,2);

	Packet* result = new Packet(&data,OP_EMULEPROT);
	result->m_eOpcode = OP_ANSWERSOURCES;
	if (nCount > 28)
		result->PackPacket();
	return result;

	EMULE_CATCH

	return NULL;
}

//For File Comment // 
void CKnownFile::LoadComment()
{
	EMULE_TRY

	CString buffer = FileHashToString(m_fileHash); 	
	
	CString fullpath; 
	fullpath.Format(_T("%sfileinfo.ini"),g_eMuleApp.m_pGlobPrefs->GetConfigDir()); 
	CIni ini( fullpath );
	ini.SetDefaultCategory(buffer);
	m_strComment = ini.GetString(_T("Comment"),_T("")); 
	m_eRating = static_cast<_EnumPartFileRating>(ini.GetInt(_T("Rate"), PF_RATING_NONE));
	m_bCommentLoaded=true;	

	ini.CloseWithoutSave();

	EMULE_CATCH
}    

void CKnownFile::RemoveFileCommentAndRating()
{
	EMULE_TRY

	CString buffer = FileHashToString(m_fileHash); 

	CString fullpath; 
	fullpath.Format(_T("%sfileinfo.ini"),g_eMuleApp.m_pGlobPrefs->GetConfigDir()); 
	CIni ini( fullpath );
	ini.DeleteCategory(buffer);
	ini.SaveAndClose();

	m_strComment = "";
	m_eRating = PF_RATING_NONE;

	EMULE_CATCH
}

void CKnownFile::SetFileComment(CString strNewComment)
{
	EMULE_TRY

	CString buffer = FileHashToString(m_fileHash); 
	       
	CString fullpath; 
	fullpath.Format(_T("%sfileinfo.ini"),g_eMuleApp.m_pGlobPrefs->GetConfigDir()); 
	CIni ini( fullpath );
	ini.SetDefaultCategory(buffer);
	    
	ini.SetString (_T("Comment"), strNewComment); 
	m_strComment = strNewComment;
	
	ini.SaveAndClose();
	   
	CTypedPtrList<CPtrList, CUpDownClient*> srclist;
	g_eMuleApp.m_pUploadQueue->FindSourcesForFileById(&srclist, GetFileHash());

	for (POSITION pos = srclist.GetHeadPosition();pos != 0;srclist.GetNext(pos)){
		CUpDownClient *cur_src = srclist.GetAt(pos);
		cur_src->SetCommentDirty();
	}

	EMULE_CATCH
}

// For File rate 
void CKnownFile::SetFileRating(EnumPartFileRating eNewRating)
{
	EMULE_TRY

	CString buffer = FileHashToString(m_fileHash); 	
	       	
	CString fullpath; 
	fullpath.Format(_T("%sfileinfo.ini"),g_eMuleApp.m_pGlobPrefs->GetConfigDir()); 
	CIni ini( fullpath );
	ini.SetDefaultCategory(buffer);
	    
	ini.SetInt (_T("Rate"), eNewRating); 
	m_eRating = eNewRating;

	ini.SaveAndClose();

	CTypedPtrList<CPtrList, CUpDownClient*> srclist;
	g_eMuleApp.m_pUploadQueue->FindSourcesForFileById(&srclist, GetFileHash());
	for (POSITION pos = srclist.GetHeadPosition();pos != 0;srclist.GetNext(pos)){
		CUpDownClient *cur_src = srclist.GetAt(pos);
		cur_src->SetCommentDirty();
	}

	EMULE_CATCH
}

//--- xrmb:parttraffic ---
void CFileStatistic::initTraffic()
{
	EMULE_TRY

	if(partTraffic==NULL)
	{
		uint16 p=fileParent->GetPartCount();
		partTraffic=new uint32[p];
		partTrafficSession=new uint32[p];
		for(uint16 i=0; i<p; i++)
		{
			partTraffic[i]=0;
			partTrafficSession[i]=0;
		}
	}

	if(partAccepted==NULL)
	{
		uint16 p=fileParent->GetPartCount();
		partAccepted=new uint32[p];
		partAcceptedSession=new uint32[p];
		for(uint16 i=0; i<p; i++)
		{
			partAccepted[i]=0;
			partAcceptedSession[i]=0;
		}
	}

	if(blockTraffic==NULL)
	{
		uint32 p=fileParent->GetBlockCount();
		blockTraffic=new uint32[p];
		blockTrafficSession=new uint32[p];
		for(uint32 i=0; i<p; i++)
		{
			blockTraffic[i]=0;
			blockTrafficSession[i]=0;
		}
	}

	EMULE_CATCH
}

void CFileStatistic::recalcCompleteReleases()
{
	EMULE_TRY

	//--- recalculate complete releases factor ---
	uint32 blocks=fileParent->GetBlockCount();
	uint32 fs=fileParent->GetFileSize();

	//--- find lowest parttraffic ---
	uint32 m=0xFFFFFFFF;
	completeReleases=0;
	for(uint32 i=0; i<blocks; i++)
	{
		if(blockTraffic[i]==0)
			continue;

		uint32 s=EMBLOCKSIZE;
		if(EMBLOCKSIZE*(i+1) > fs) 
			s = fs % EMBLOCKSIZE; 
		
		if(s)
		{
			uint32 f=(uint32)(double)blockTraffic[i]/s+(blockTraffic[i]==s?0:1);
			if(f<m) m=f;
		}
	}

	//--- get avg. ---
	for(uint32 i=0; i<blocks; i++)
	{
		if(blockTraffic[i]==0)
			continue;

		uint32 s=EMBLOCKSIZE;
		if(EMBLOCKSIZE*(i+1) > fs) 
			s = fs % EMBLOCKSIZE;

		if(s)
		{
			double f=(double)blockTraffic[i]/s;
			if(f>m) f=m;
			completeReleases+=f;
		}
	}

	completeReleases/=blocks;

	EMULE_CATCH
}

//--- this function may not return a 100% accurate value, because you can't fill a part with blocks
double CFileStatistic::GetCompletePartReleases(uint16 part)
{
	EMULE_TRY

	//--- check part, damn that all needs time, why we just cant trust the devs :) ---
	if(partTraffic==NULL || part<0 || part>fileParent->GetPartCount() || partTraffic[part]==0)
		return 0;

	//--- start and end block ---
	uint32	startBlock=(part*PARTSIZE)/EMBLOCKSIZE;
	uint32	endBlock=((part+1)*PARTSIZE)/EMBLOCKSIZE;
	if(endBlock >= fileParent->GetBlockCount())
		endBlock=fileParent->GetBlockCount()-1;

	uint32	fs=fileParent->GetFileSize();

	//--- find lowest parttraffic ---
	uint32	m=0xFFFFFFFF;
	double	completePartReleases=0;
	for(uint32 i=startBlock; i<=endBlock; i++)
	{
		if(blockTraffic[i]==0)
			continue;

		uint32 s=EMBLOCKSIZE;
		if(EMBLOCKSIZE*(i+1) > fs) 
			s = fs % EMBLOCKSIZE; 
		
		if(s)
		{
			uint32 f=(uint32)(double)blockTraffic[i]/s+(blockTraffic[i]==s?0:1);
			if(f<m) m=f;
		}
	}

	//--- get avg. ---
	for(uint32 i=startBlock; i<=endBlock; i++)
	{
		if(blockTraffic[i]==0)
			continue;

		uint32 s=EMBLOCKSIZE;
		if(EMBLOCKSIZE*(i+1) > fs) 
			s = fs % EMBLOCKSIZE;

		if(s)
		{
			double f=(double)blockTraffic[i]/s;
			if(f>m) f=m;
			completePartReleases+=f;
		}
	}

	return completePartReleases/(fileParent->GetPartSize(part)/EMBLOCKSIZE);

	EMULE_CATCH

	return 0;
}

void CFileStatistic::AddTraffic(uint32 start, uint32 bytes)
{
	EMULE_TRY

	initTraffic();

	//DEBUG_ONLY(AddLogLine(false, "--------------------------------------------------------------------------------------------------"));
	//DEBUG_ONLY(AddLogLine(false, "start %i, bytes %i ", start, bytes));

	//--- start block ---
	uint32	part=start/PARTSIZE;
	sint32	btg=(sint32)bytes;

	uint32 p1=min((part+1)*PARTSIZE-start, bytes);
	partTraffic[part]+=p1;
	partTrafficSession[part]+=p1;
	partAccepted[part]++;
	partAcceptedSession[part]++;
	btg-=p1;
	part++;
	while(btg>0)
	{
		partTraffic[part]+=btg % PARTSIZE;
		partTrafficSession[part]+=btg % PARTSIZE;
		partAccepted[part]++;
		partAcceptedSession[part]++;
		part++;
		btg-=PARTSIZE;
	}

	//--- start block ---
	uint32	block=start/EMBLOCKSIZE;
	btg=(sint32)bytes;

	uint32 b1=min((block+1)*EMBLOCKSIZE-start, bytes);
	blockTraffic[block]+=b1;
	blockTrafficSession[block]+=b1;
	btg-=b1;
	block++;
	while(btg>0)
	{
		blockTraffic[block]+=btg % EMBLOCKSIZE;
		blockTrafficSession[block]+=btg % EMBLOCKSIZE;
		block++;
		btg-=EMBLOCKSIZE;
	}

	recalcCompleteReleases();
	
	fileParent->UpdateSharedFileDisplay();

	EMULE_CATCH
}

bool CKnownFile::SaveToFileTraffic(FILE* file)
{
	EMULE_TRY

	if(statistic.partTraffic)
	{
		uint16 parts=GetPartCount();
		if(fwrite(&parts, sizeof(parts), 1, file)!=1) return false;
		if(fwrite(statistic.partTraffic, 4, parts, file)!=parts) return false;

		uint32 blocks=GetBlockCount();
		if(fwrite(&blocks, sizeof(blocks), 1, file)!=1) return false;
		if(fwrite(statistic.blockTraffic, 4, blocks, file)!=blocks) return false;

		//--- added in version 2 ---
		if(fwrite(statistic.partAccepted, 4, parts, file)!=parts) return false;
	}

	//--- this block shouldnt be used at all ---
	else
	{
		uint32	d=0;

		uint16 parts=GetPartCount();
		if(fwrite(&parts, sizeof(parts), 1, file)!=1) return false;
		for(uint16 part=0; part<parts; part++)
			if(fwrite(&d, sizeof(d), 1, file)!=1) return false;

		uint32 blocks=GetBlockCount();
		if(fwrite(&blocks, sizeof(blocks), 1, file)!=1) return false;
		for(uint32 block=0; block<blocks; block++)
			if(fwrite(&d, sizeof(d), 1, file)!=1) return false;

		//--- added version 2 ---
		for(uint16 part=0; part<parts; part++)
			if(fwrite(&d, sizeof(d), 1, file)!=1) return false;
	}

	return ferror(file);

	EMULE_CATCH

	return false;
}

bool CKnownFile::LoadFromFileTraffic(FILE* file, uint8 version)
{
	EMULE_TRY

	// read parts
	uint16 parts=0;
	if(fread(&parts, sizeof(parts), 1, file)!=1) return false;
	
	// does part count match?
	if(parts!=GetPartCount())
	{
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_TRAFFICDAT_CORRUPT));
		return false;
	}

	// get the memory
	statistic.initTraffic();

	// load all the parts traffic data
	if(fread(statistic.partTraffic, 4, parts, file)!=parts) return false;

	// read blocks
	uint32 blocks=0;
	if(fread(&blocks, sizeof(blocks), 1, file)!=1) return false;
	
	// does blocks count match?
	if(blocks!=GetBlockCount())
	{
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_TRAFFICDAT_CORRUPT));
		return false;
	}

	// load all the parts traffic data
	if(fread(statistic.blockTraffic, 4, blocks, file)!=blocks) return false;

	//--- versioning ---
	if(version>1)
	{
		//--- version 2 introduced part-accepted counter ---
		if(fread(statistic.partAccepted, 4, parts, file)!=parts) return false;
	}

	//--- recalc something ---
	statistic.recalcCompleteReleases();

	return true;

	EMULE_CATCH

	return false;
}

uint32 CKnownFile::GetBlockCount()
{
	EMULE_TRY

	return (m_lFileSize/EMBLOCKSIZE) + ((m_lFileSize % EMBLOCKSIZE)? 1:0);

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetTrafficBlock(uint32 start, uint32 end, bool session)
{
	EMULE_TRY

	if(statistic.blockTraffic==NULL)
		return 0;

	if(start>m_lFileSize) return 0;
	if(end>m_lFileSize) end=m_lFileSize;

	uint32	*data;
	if(session)
		data=statistic.blockTrafficSession;
	else
		data=statistic.blockTraffic;

	uint32	block=start/EMBLOCKSIZE;
	uint32	pos=start;
	sint32	bftb;
	uint32	t=0;
	while(pos < end)
	{
		if(end-pos > end % EMBLOCKSIZE)
			bftb=EMBLOCKSIZE;
		else
			bftb=end % EMBLOCKSIZE;
		bftb-=pos % EMBLOCKSIZE;

		if(bftb != EMBLOCKSIZE)
			t+=data[block] * (double)bftb/GetBlockSize(block);
		else
			t+=data[block];

		pos+=bftb;

		block++;
	}

	return t;

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetTrafficPart(uint32 start, uint32 end, bool session)
{
	EMULE_TRY

	if(statistic.partTraffic==NULL)
		return 0;

	if(start>m_lFileSize) return 0;
	if(end>m_lFileSize) end=m_lFileSize;

	uint32	*data;
	if(session)
		data=statistic.partTrafficSession;
	else
		data=statistic.partTraffic;

	uint32	part=start/PARTSIZE;
	uint32	pos=start;
	sint32	bftp;
	uint32	t=0;
	while(pos < end)
	{
		if(end-pos > end % PARTSIZE)
			bftp=PARTSIZE;
		else
			bftp=end % PARTSIZE;
		bftp-=pos % PARTSIZE;

		if(bftp != PARTSIZE)
			t+=data[part] * (double)bftp/GetPartSize(part);
		else
			t+=data[part];

		pos+=bftp;

		part++;
	}

	return t;

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetPartTraffic(uint16 part, bool session)
{
	EMULE_TRY

	if(statistic.partTraffic==NULL)
		return 0;

	if(session)
		return statistic.partTrafficSession[part];
	else
		return statistic.partTraffic[part];

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetBlockTraffic(uint16 block, bool session)
{
	EMULE_TRY

	if(statistic.partTraffic==NULL)
		return 0;

	if(session)
		return statistic.blockTrafficSession[block];
	else
		return statistic.blockTraffic[block];

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetPartSize(uint16 part)
{
	EMULE_TRY

	if(part+1<GetPartCount())
		return PARTSIZE;

	return m_lFileSize % PARTSIZE;

	EMULE_CATCH

	return 0;
}

uint32 CKnownFile::GetBlockSize(uint32 block)
{
	EMULE_TRY

	if(block+1<GetBlockCount())
		return EMBLOCKSIZE;

	return m_lFileSize % EMBLOCKSIZE;

	EMULE_CATCH

	return 0;
}

void CFileStatistic::resetStats(bool all)
{
	EMULE_TRY

	if(all)
	{
		if(partTraffic)		delete[] partTraffic;	partTraffic=NULL;
		if(partAccepted)	delete[] partAccepted;	partAccepted=NULL;
		if(blockTraffic)	delete[] blockTraffic;	blockTraffic=NULL;
	
		if(partTrafficSession)		delete[] partTrafficSession;	partTrafficSession=NULL;
		if(partAcceptedSession)		delete[] partAcceptedSession;	partAcceptedSession=NULL;
		if(blockTrafficSession)		delete[] blockTrafficSession;	blockTrafficSession=NULL;

		completeReleases=0.0;

		alltimerequested=alltimetransferred=alltimeaccepted=0;
	}
	else
	{
		if(partTrafficSession)
			for(uint16 part=0; part<fileParent->GetPartCount(); part++)
				partTrafficSession[part]=0;

		if(partAcceptedSession)
			for(uint16 part=0; part<fileParent->GetPartCount(); part++)
				partAcceptedSession[part]=0;

		if(blockTrafficSession)
			for(uint32 block=0; block<fileParent->GetBlockCount(); block++)
				blockTrafficSession[block]=0;
	}
	
	m_iNumRequested=m_qwNumTransferred=m_iNumAccepted=0;

	EMULE_CATCH
}

void CFileStatistic::resetPartTraffic(uint16 part, bool all)
{
	EMULE_TRY

	if(partTraffic==NULL || part>=fileParent->GetPartCount())
		return;

	//--- this all is a little bit tricky, because the part and block size have no connections ---
	//--- so i have to remove some bytes only from the start and end block ---
	//--- all we try is to keep it as realistic as possible :) ---
	uint32	blocks=fileParent->GetBlockCount();
	uint32	blockStart=(part*PARTSIZE)/EMBLOCKSIZE;
	uint32	blockEnd=min((((uint32)part+1)*PARTSIZE)/EMBLOCKSIZE+1, blocks-1);
	double	percentStart=(static_cast<double>((part*PARTSIZE)%EMBLOCKSIZE)/EMBLOCKSIZE);
	double	percentEnd=0.0;
	if(blockEnd!=blocks)
		percentEnd=1-(static_cast<double>(((part+1)*PARTSIZE)%EMBLOCKSIZE)/EMBLOCKSIZE);

	if(all)
	{
		partTraffic[part]=0;
		blockTraffic[blockStart]*=percentStart;
		blockTraffic[blockEnd]*=percentEnd;
		for(uint32 block=blockStart+1; block<blockEnd; block++)
			blockTraffic[block]=0;
	}

	partTrafficSession[part]=0;
	blockTrafficSession[blockStart]*=percentStart;
	blockTrafficSession[blockEnd]*=percentEnd;
	for(uint32 block=blockStart+1; block<blockEnd; block++)
		blockTrafficSession[block]=0;

	EMULE_CATCH
}

uint32	CFileStatistic::GetPartAccepted(uint16 part, bool session) const
{
	EMULE_TRY

	if(partTraffic==NULL || part>=fileParent->GetPartCount())
		return 0;

	if(session==false)
		return partAccepted[part];
	else
		return partAcceptedSession[part];

	EMULE_CATCH

	return 0;
}
//--- :xrmb ---

//--- xrmb:keepPartFileStats ---
void CFileStatistic::AddAll(uint32 _alltimerequested, uint32 _alltimeaccepted, uint64 _alltimetransferred)
{
	alltimerequested	+= _alltimerequested;
	alltimeaccepted		+= _alltimeaccepted;
	alltimetransferred	+= _alltimetransferred;
}
//--- :xrmb ---

//--- xrmb:allyaknow ---
//-------------------------------------------------------------------------------------------------
bool CFileStatistic::merge(CFileStatistic *m)
{
	EMULE_TRY

	alltimerequested	+= m->alltimerequested;
	alltimeaccepted		+= m->alltimeaccepted;
	alltimetransferred	+= m->alltimetransferred;

	m_iNumRequested	+= m->m_iNumRequested;
	m_iNumAccepted	+= m->m_iNumAccepted;
	m_qwNumTransferred	+= m->m_qwNumTransferred;

	//--- merge parttraffic ---

	//--- i dont have traffic, but the to merge has ---
	if(partTraffic==NULL && m->partTraffic)
		initTraffic();

	//--- merge traffic only if the to merge has some ---
	if(m->partTraffic)
	{
		for(uint16 part=0; part<fileParent->GetPartCount(); part++)
			partTraffic[part]=m->partTraffic[part];

		for(uint32 block=0; block<fileParent->GetBlockCount(); block++)
			blockTraffic[block]+=m->blockTraffic[block];
	}

	return true;

	EMULE_CATCH

	return false;
}
//--- :xrmb ---

//--- xrmb:partprio ---
uint8	CKnownFile::GetPartStatus(uint16 part)
{ 
	EMULE_TRY

	uint8 r=PR_PART_ON; 
	if(m_partStatus.Lookup(part, r)) 
		return r; 

	EMULE_CATCH

	return PR_PART_ON; 
}

bool    CKnownFile::HasParts()
{
	EMULE_TRY

	POSITION    pos=m_partStatus.GetStartPosition();

	if (pos == NULL)
		return true;

	uint16      part;
	uint8       status;
	while(pos)
	{
		m_partStatus.GetNextAssoc(pos, part, status);
		if (status!=PR_PART_OFF)
			return true;
	}

	EMULE_CATCH

	return false;
}

bool	CKnownFile::HasHiddenParts()
{
	EMULE_TRY

	POSITION	pos=m_partStatus.GetStartPosition();
	uint16		part;
	uint8		status;
	while(pos)
	{
		m_partStatus.GetNextAssoc(pos, part, status);
		if(status!=PR_PART_ON)
			return true;
	}

	EMULE_CATCH

	return false;
}

void CKnownFile::WritePartStatus(CFile* file)
{
	EMULE_TRY
	//Cax2 - this lock here was pretty useless..
	//CSingleLock Lock(&m_mutexHashList,TRUE);
	uint16 parts = m_partHashArray.GetCount();
	//Lock.Unlock();
	file->Write(&parts, 2);
	uint16 done = 0;
	while(done!=parts)
	{
		uint8 towrite = 0;
		for (uint32 i = 0;i != 8;i++)
		{
			if (GetPartStatus(done)==PR_PART_ON)
				towrite |= (1<<i);

			done++;
			if (done == parts)
				break;
		}
		file->Write(&towrite,1);
	}

	EMULE_CATCH
}
//--- :xrmb/partprio ---

// obaldin
#if JUMPSTART

bool CKnownFile::IsJsComplete()
{
     return CJumpstarter::IsJsCompleteForFile(this);
}

void CKnownFile::SetJumpstartEnabled(bool enabled)
{
    if(!enabled && m_Jumpstarter)
	{
        m_Jumpstarter->Disable();
        delete m_Jumpstarter;
        m_Jumpstarter = NULL;
    }
	else if(enabled && !m_Jumpstarter)
	{
        CJumpstarter::EnableForFile(this);
        m_Jumpstarter = new CJumpstarter(this);
    }
}
void CKnownFile::AddSentBlock(CUpDownClient* client,uint32 start_offset, uint32 togo)
{
    m_Jumpstarter->AddSentBlock(client, start_offset, togo);
}

void CKnownFile::WriteJumpstartPartStatus(CUpDownClient* client, CMemFile* data)
{
    m_Jumpstarter->WriteJumpstartPartStatus(client, data);
}

bool CKnownFile::AllowChunkForClient(uint32 partNo, CUpDownClient* client)
{
    return (!m_Jumpstarter) || m_Jumpstarter->AllowChunkForClient(partNo, client);
}

#endif
