//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#pragma once
#include "searchlistctrl.h"
#include "afxcmn.h"
#include "packets.h"
#include "types.h"
#include "afxwin.h"
#include "iconstatic.h"
#include "closabletabctrl.h"
#include "Loggable.h"

class CCustomAutoComplete;

typedef struct 
{
	CString sNotSearch;
	bool bDocumentSearch;
} SearchData;
typedef CMap<int, int, SearchData, SearchData> SearchMap;

// CSearchDlg dialog
class CSearchDlg : public CResizableDialog, public CLoggable
{
	DECLARE_DYNAMIC(CSearchDlg)

public:
	CSearchDlg(CWnd* pParent = NULL);   // standard constructor
	virtual ~CSearchDlg();
	
	afx_msg void OnBnClickedStarts();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnBnClickedCancels();
	afx_msg void OnBnClickedSdownload();
	afx_msg void OnBnClickedClearall();
	afx_msg void OnChangeMax();
	afx_msg void OnChangeMin();
	afx_msg void OnSearchKeyDown();

	enum { IDD = IDD_SEARCH };

	void	Localize();
	void	DownloadSelected();
	void	LocalSearchEnd(uint16 count);
	void	AddUDPResult(uint16 count);
	void	DeleteSearch(uint32 nSearchID);
	void	DeleteAllSearchs();
	void	CreateNewTab(CString searchString,uint32 nSearchID);
	void	UpdateCatTabs();
	void	OnChangeSize(bool changedMin);		//Cax2
	void	AddEd2kLinksToDownload(CString strlink,EnumCategories eCatID);
	void	AddEd2kLinksToDownload(CString strlink)	{AddEd2kLinksToDownload(strlink,CCat::GetCatIDByIndex(m_ctrlCatTabs.GetCurSel()));}
	CSearchListCtrl m_ctlSearchList;
	CClosableTabCtrl m_ctlSearchTabs;
	CString	ToQueryString(CString str); //Cax2
	void	DoNewSearch(CString searchString, CString typeText, unsigned long min, unsigned long max, 
			int availability, CString extension, bool doGlobal);
    CString	GetNotSearch(int nSearchId);
	bool	IsDocumentSearch(int nSearchId);
	BOOL	SaveSearchStrings();
	
protected:
	void StartNewSearch();
	LRESULT OnCloseTab(WPARAM wparam, LPARAM lparam);
	LRESULT OnShowWindow(WPARAM wparam, LPARAM lparam);
	CString	CreateWebQuery();

	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual BOOL OnInitDialog();
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	afx_msg void OnNMDblclkSearchlist(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnTcnSelchangeTab1(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnEnChangeSearchname();
	afx_msg void OnEnChangeMethod();

	DECLARE_MESSAGE_MAP()

private:
	Packet*		pSearchPacket;		
	UINT_PTR	global_search_timer;
	CProgressCtrl searchprogress;
	bool		canceled;
	uint16		servercount;
	bool		globsearch;
	uint32		m_nSearchID;
	CComboBoxEx	methodbox;
	CComboBox	Stypebox;
	CImageList	m_pImageList;
	CTabCtrl	m_ctrlCatTabs;
	CImageList	m_BoxImageList;
	CIconStatic m_ctrlSearchFrm;
	CIconStatic m_ctrlDirectDlFrm;
	CCustomAutoComplete* m_pacSearchString;
	//Resources cleanup - [TwoBottle Mod]
	HICON oldSearchLstIcon;
	CStatic* GetSearchLstStatic() const { return (CStatic*)GetDlgItem(IDC_SEARCHLST_ICO); }
	HICON GetSearchLstIcon() const { return (HICON)::LoadImage(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_SEARCHRESULTS), IMAGE_ICON, 16, 16, 0); }
	SearchMap m_SearchMap;

	// constants definition
	static const long typeNemonic = 0x00030001;
	static const long extensionNemonic = 0x00040001;
	static const long availabilityNemonic = 0x15000101;
	static const long minNemonic = 0x02000101;
	static const long maxNemonic = 0x02000102;

	static const byte stringParameter=1;
	static const byte typeParameter=2;
	static const byte numericParameter=3;

	static const uint16 andParameter=0x0000;
	static const uint16 orParameter=0x0100;
public:
	afx_msg void OnBnClickedSearchReset();
	afx_msg void OnStnClickedSearchextensionLbl();
	afx_msg void OnBnClickedDownloadAllEd2k();
	afx_msg void OnDestroy();
};
