//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "UploadQueue.h"
#include "packets.h"
#include "emule.h"
#include "knownfile.h"
#ifdef OLD_SOCKETS_ENABLED
#include "ListenSocket.h"
#endif //OLD_SOCKETS_ENABLED
#include "ini2.h"
#include "math.h"
#include "Smtp.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


static uint32	g_iCounter, g_iSec, g_iStatSave;
// -khaos--+++> Added g_iUpdateConnStats...
static uint32	g_iGraph, g_iStats, g_iUpdateConnStats;
// <-----khaos-

VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg,UINT_PTR idEvent,DWORD dwTime);
UINT SaveSourcesThread(HANDLE lpArg);

//	TODO rewrite the whole networkcode, use overlapped sockets

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUploadQueue::CUploadQueue(CPreferences* in_prefs)
{
	EMULE_TRY

	m_pAppPrefs = in_prefs;
	m_bExiting	=	FALSE;
	m_hTimer = SetTimer(NULL,141,100,TimerProc); //TODO: What event is 141?
	if (!m_hTimer)
		AddDebugLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_TIMERCREATEFAILED));
	m_iEstMaxDataRate = 2000;
	m_iDataRate = 0;
	m_iDataRateAvg = 0;
	m_iBannedCount = 0;
	g_iCounter=0;
//	sumavgdata = 0;				// enkeyDEV(Ottavio84) -Accurate speed measurement-
//	sendtimestamp = GetTickCount();	// enkeyDEV(Ottavio84) -Upload rate is made independent from the inaccurate WM_TIMER-
//	bDataRateMaxed = false; 		// Tarod [LoneStar]
//	lasttimechecked = GetTickCount() ;
	//eklmn: Extended Upload stats
	m_iULSessionSuccessful = 0;
	m_iULSessionSuccessfulFullChunk = 0;
	for (int i=0; i<ETS_TERMINATOR; i++)
	{
		m_iULSessionSuccessfulPartChunk[i] = 0;
		m_iULSessionFailed[i] = 0;
	}
	m_iTotalUploadTime = 0;
	m_nUpDataRateMSOverhead = 0;
	m_nUpDataRateOverhead = 0;
	m_nUpDataOverheadSourceExchange = 0;
	m_nUpDataOverheadFileRequest = 0;
	m_nUpDataOverheadOther = 0;
	m_nUpDataOverheadServer = 0;
	m_nUpDataOverheadSourceExchangePackets = 0;
	m_nUpDataOverheadFileRequestPackets = 0;
	m_nUpDataOverheadOtherPackets = 0;
	m_nUpDataOverheadServerPackets = 0;
	g_iStatSave=0;
	// -khaos--+++>
	g_iUpdateConnStats=0;
	// <-----khaos-
    for (int i = 0;i != 35;i++)
		activeClients_list.AddHead((int)0);

    m_MaxActiveClients = 0;

    m_iLeftOverBandwidth = 0;

    m_guessedMaxBandwidth = 300;
	m_guessedMaxLANBandwidth = 3000;

    m_lastGaveDataTick = ::GetTickCount();
    m_lastCalculatedDataRateTick = 0;

	// Create timer to save known sources each 30 min 
	m_hSaveSTimer	=	::CreateWaitableTimer(NULL,FALSE,NULL);
	if ( m_hSaveSTimer )
	{
	//	Create lower priority thread that will do the "job"
		m_pSaveThread = AfxBeginThread( SaveSourcesThread,
										(LPVOID)this,
										THREAD_PRIORITY_NORMAL+g_eMuleApp.m_pGlobPrefs->GetMainProcessPriority() );
		if ( ! m_pSaveThread )
		{
			::CloseHandle(m_hSaveSTimer);
			m_hSaveSTimer	=	NULL;
			AddDebugLogLine(false, RGB_LOG_ERROR + _T("Error creating SourcesSaveThread"));
		}
	}
	else
	{
		AddDebugLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_TIMERCREATEFAILED));
	}

	EMULE_CATCH2
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUploadQueue::AddUpNextClient(CUpDownClient* directadd)
{
	EMULE_TRY
#ifdef OLD_SOCKETS_ENABLED
	if (g_eMuleApp.m_pListenSocket->TooManySockets()) return;
#endif //OLD_SOCKETS_ENABLED
	POSITION toadd = 0;
	uint32	bestscore = 0;
	CUpDownClient* newclient;

	// select next client or use given client
	if (!directadd)
	{
		POSITION pos1, pos2;
		CUpDownClient* cur_client;
		
		for (pos1 = waitinglist.GetHeadPosition();( pos2 = pos1 ) != NULL;)
		{
			waitinglist.GetNext(pos1);
			cur_client  = waitinglist.GetAt(pos2);
			
			// clear dead clients
			ASSERT(cur_client->GetLastUpRequest());

			if ( (::GetTickCount() - cur_client->GetLastUpRequest() > MAX_PURGEQUEUETIME)
				|| !g_eMuleApp.m_pSharedFilesList->GetFileByID(cur_client->m_reqFileHash) )   // InterCeptor (remove "?" from queue) - readded by DonGato
			{
				RemoveFromWaitingQueue(pos2,true);
#ifdef OLD_SOCKETS_ENABLED
				if(!cur_client->m_pRequestSocket)
				{
					if (cur_client->Disconnected())
						cur_client = NULL;
				}
#endif //OLD_SOCKETS_ENABLED
				continue;	// eklmn: bugfix(17): upload fix(jump to next client)
			}
			// finished clearing
			else if ( (cur_client->GetScore(true) > bestscore) && (!cur_client->IsBanned())  && (!cur_client->HasLowID()
#ifdef OLD_SOCKETS_ENABLED
				|| (cur_client->m_pRequestSocket && cur_client->m_pRequestSocket->IsConnected()) 
#endif //OLD_SOCKETS_ENABLED
				) )
			{
				bestscore = cur_client->GetScore(true);
				toadd = pos2;
			}
		}
		if (!toadd)
			return;
		newclient = waitinglist.GetAt(toadd);
		RemoveFromWaitingQueue(toadd, true);
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateUploadHeader(); // changed by FoRcHa
	}
	else
		newclient = directadd;

	// obaldin: bug found by dr.clone, but it could be deeper
  	// let's check of client existence in ClientList
  	if(!g_eMuleApp.m_pClientList->IsValidClient(newclient))
	{
		AddDebugLogLine(false, _T("AddUpNextClient(): client not in ClientList"));
		return;
	}

	//SyruS: never upload to a banned & already downloading client
	if (IsDownloading(newclient) || newclient->IsBanned(true)) 	
		return;

	// eklmn: Don't accept clients if requested file is in PS_COMPLETING state to prevent unpredictable behaviour 
	// (the existence of the requested file will be done in AddClientToQueue)
	CKnownFile* m_pReqPartFile = g_eMuleApp.m_pSharedFilesList->GetFileByID((uchar*)newclient->m_reqFileHash);
	if (m_pReqPartFile->IsPartFile() && ((CPartFile*)m_pReqPartFile)->GetStatus() == PS_COMPLETING)
		return;
	
	// tell the client that we are now ready to upload
#ifdef OLD_SOCKETS_ENABLED
	if (!newclient->m_pRequestSocket || !newclient->m_pRequestSocket->IsConnected())
	{
		newclient->SetUploadState(US_CONNECTING);
		if (!newclient->TryToConnect(true))	// SLUGFILLER: SafeDisconnect
			return;
	}
	else
	{
		Packet* packet = new Packet(OP_ACCEPTUPLOADREQ);
		g_eMuleApp.m_pUploadQueue->AddUpDataOverheadFileRequest(packet->m_dwSize);
		newclient->m_pRequestSocket->SendPacket(packet,true);
		newclient->SetUploadState(US_UPLOADING);
		newclient->SetLastGotULData();	//eklmn: set Upload Timer
	}
#endif //OLD_SOCKETS_ENABLED
	
/*
	if(ASK_FOR_PARTS_ON_UPLOAD)
		newclient->SendFileRequestForUpload();
*/
	newclient->SetUpStartTime();
	newclient->ResetSessionUp();
	newclient->ResetCompressionGain(); 	// Add show compression
	uploadinglist.AddTail(newclient);
	
	// statistic
	//CKnownFile* m_pReqPartFile = g_eMuleApp.m_pSharedFilesList->GetFileByID((uchar*)newclient->m_reqFileHash);
	if (m_pReqPartFile)
		m_pReqPartFile->statistic.AddAccepted();
	
	g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlUploadList.AddClient(newclient);

	EMULE_CATCH
}

void CUploadQueue::Process()
{
	EMULE_TRY

    DWORD curTick = ::GetTickCount();
    int timeSinceLastGaveDataMs;

	// LANCAST (moosetea) - We count lanclients
	int QtyLANClients = 0;

    // after 49 days tick wraps around. I think this still handles it. (Yeah right! As if we wouldn't have crashed by then :))
    timeSinceLastGaveDataMs = curTick - m_lastGaveDataTick;

    // Keeps track of how much bandWidth we use this call
    sint32 usedBandwidthThisCall = 0;

    // How much data should be given each round?
	if(g_eMuleApp.m_pGlobPrefs->GetMaxUpload() == UNLIMITED) {
        // Special case for unlimited upload

        if (CanAcceptNewClient(uploadinglist.GetCount()) && waitinglist.GetCount()) {
		    AddUpNextClient();
        }

        bool clientWasReady = false;

	    POSITION pos = uploadinglist.GetHeadPosition();
	    while (pos != NULL){
            // Get the client. Note! Also updates activeClientsListPos as a side effect.
		    CUpDownClient* cur_client = uploadinglist.GetNext(pos);
            bool tempWantsMoreBandwidth;

			// LANCAST (moosetea) - Just Count LAN Clients
			if (cur_client->IsOnLAN())
				QtyLANClients++;
			else
				usedBandwidthThisCall += cur_client->SendBlockData(m_guessedMaxBandwidth-usedBandwidthThisCall, tempWantsMoreBandwidth);

            if(tempWantsMoreBandwidth == true) {
                clientWasReady = true;
            }
	    }

	    if (clientWasReady == false){
            if(m_guessedMaxBandwidth > 300) {
		        m_guessedMaxBandwidth -= 200;
            } else {
                m_guessedMaxBandwidth = 100;
            }
	    } else {
		    m_guessedMaxBandwidth += 200;
	    }

        m_MaxActiveClients = uploadinglist.GetCount();
    } else {
        // Normal case for limited upload.

        uint32 tempMaxActiveClients = 0;
        POSITION activeClientsListPos = activeClients_list.GetHeadPosition();
        while(activeClientsListPos != NULL) {
            uint32 activeClientsSnapshot = activeClients_list.GetAt(activeClientsListPos);

            if(activeClientsSnapshot > tempMaxActiveClients) {
                tempMaxActiveClients = activeClientsSnapshot;
            }

            activeClients_list.GetNext(activeClientsListPos);
        }
        m_MaxActiveClients = tempMaxActiveClients;

	    sint32 realMaxULRatems = (g_eMuleApp.m_pGlobPrefs->GetMaxUpload()*1024*timeSinceLastGaveDataMs)/1000;

#define MINNUMBEROFTRICKLEUPLOADS 2

        // How many slots should be open? Trickle slots included (at least 2 trickles, 30% of total, and no slot is expected to use more than UPLOAD_CLIENT_DATARATE*2 in average, whichever number is largest)
        sint32 wantedNumberOfTotalUploads = max((uint32)(g_eMuleApp.m_pGlobPrefs->GetMaxUpload()*1024/(UPLOAD_CLIENT_DATARATE*2)), m_MaxActiveClients + MINNUMBEROFTRICKLEUPLOADS);
        wantedNumberOfTotalUploads = max(wantedNumberOfTotalUploads, m_MaxActiveClients*1.3);

		POSITION lastpos = uploadinglist.GetTailPosition();

        CUpDownClient* lastClient = NULL;
        if(lastpos != NULL) {
            lastClient = uploadinglist.GetAt(lastpos);
        }

#define MINWAITBEFOREOPENANOTHERSLOTMS 1000

        // Add or remove connections as needed. Mostly, this is controlled by the number of
        // fully active uploads.

        if(CanAcceptNewClient(uploadinglist.GetCount()) && waitinglist.GetCount() > 0 &&
           wantedNumberOfTotalUploads > uploadinglist.GetCount() &&
           (lastClient == NULL || lastClient->GetUpStartTimeDelay() > MINWAITBEFOREOPENANOTHERSLOTMS)) {
            // There's not enough open uploads. Open more another one.
            AddUpNextClient();
        } else if(uploadinglist.GetCount() > MIN_UP_CLIENTS_ALLOWED &&
                CanAcceptNewClient(uploadinglist.GetCount()-1) == false) {
            // There's to many open uploads (propably due to the user changing
            // the upload limit to a lower value). Remove the last opened upload and put
            // it back on the waitinglist. When it is put back, it get
            // to keep its waiting time. This means it is likely to soon be
            // choosen for upload again.
            if(!lastClient->IsOnLAN() && lastClient->GetUpStartTimeDelay() > 3*1000) {   
		        // Remove from upload list.
		        g_eMuleApp.m_pUploadQueue->RemoveFromUploadQueue(lastClient, ETS_TIMEOUT, true);

		        // add to queue again. This is almost the same as in method
		        //    g_eMuleApp.m_pUploadQueue->AddClientToQueue(lastClient,true);
		        waitinglist.AddTail(lastClient);
		        lastClient->SetUploadState(US_ONUPLOADQUEUE);
		        lastClient->SendRankingInfo();
		        g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlQueueList.AddClient(lastClient, false);
		        g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateUploadHeader();
            }
        }

        // There's some saved bandwidth that we want to try to spend this round.
        // curMaxULRatems keeps track of how much data we want to spend.
        sint32 curMaxULRatems = realMaxULRatems + m_iLeftOverBandwidth;

        sint32 allowedBandwidthThisCall = min(curMaxULRatems, realMaxULRatems+MAXFRAGSIZE);

        // Which active slot are we giving data each loop
        uint32 slotcounter = 0;

        bool wantsMoreBandWidth = false;

        POSITION ulpos = uploadinglist.GetHeadPosition();
        // The loop that gives the fully activated connections data.
        while (ulpos != NULL && /*slotcounter < activeClients &&*/ (allowedBandwidthThisCall > usedBandwidthThisCall) && wantsMoreBandWidth == false) {
            // Only allow this connection to take what has not already been taken
            // by the other connections above it in the list.
            uint32 allowedThisIteration = allowedBandwidthThisCall-usedBandwidthThisCall;

            // Get the client. Note! Also updates ulpos as a side effect.
		    CUpDownClient* cur_client = uploadinglist.GetNext(ulpos);

            // Give data to the upload, if it wants it. Remember how much data we have given this call.
            // The connection wont take any data if it hasn't emptied its buffer since we last gave it data.
            // variable wantsMoreBandWidth is updated by this method (it is a ref bool&)

			// LANCAST (moosetea) - Increment LanCount
			if (cur_client->IsOnLAN())
				QtyLANClients++;
			else
				usedBandwidthThisCall += cur_client->SendBlockData(allowedThisIteration, wantsMoreBandWidth);

            slotcounter++;
	    }

#define MAXTIMESLOTISALLOWEDTOGOWITHOUTDATAMS 2540
        // Trickle the unneeded uploads (just give them enough to not time out)
        // Theese downloads are kept connected, in a ready-to-go state, just in case
        // one of the fully activated uploads completes/timeouts/ends.
        // As soon as there's a little bandwidth leftover, the first one of these
        // uploads will go to fully activated state
        POSITION trickle_client_pos = uploadinglist.FindIndex(slotcounter);
        while (trickle_client_pos != NULL){
            // Get the client. Note! Also updates trickle_client_pos as a side effect.
	        CUpDownClient* cur_client = uploadinglist.GetNext(trickle_client_pos);

            if(cur_client->GetLastGotULData() + MAXTIMESLOTISALLOWEDTOGOWITHOUTDATAMS < curTick) {
                // It's more than 3 seconds since this connection got any data.
                // Feed it a mercy package to prevent it from timing out.
                bool tempWantsMoreData = false;
				// LANCAST (moosetea) - Increment LanCount
				if (cur_client->IsOnLAN())
					QtyLANClients++;
				else
					usedBandwidthThisCall += cur_client->SendBlockData(MAXFRAGSIZE, tempWantsMoreData);
            }
        }

        // Since we don't save bandwidth for the trickles above, we may
        // have used to much bandwidth.
        // We may also have used to little bandwidth this round, in that
        // case it is saved to the next round.
        m_iLeftOverBandwidth = curMaxULRatems-usedBandwidthThisCall;
        
        // We don't want to long peaks. Don't save to much bandwidth (about a second should be OK)
        // Always make sure we can save enough to get a packet through
        sint32 limitSave = MAXFRAGSIZE; // max(realMaxULRatems*10, MAXFRAGSIZE+realMaxULRatems*4);
        if(m_iLeftOverBandwidth > limitSave) {
            m_iLeftOverBandwidth = limitSave;
        }

        // Save number of active clients for statistics
   	    activeClients_list.RemoveHead();
	    activeClients_list.AddTail(slotcounter);
    }

    // Save sent bytes for speed calculations
	//Cax2 - refactored these lists: easier access to a subset of the data, and less memory usage!

	if(m_averageTickList.empty() || curTick- m_averageTickList.front()>=500)	//Cax2 - update no faster than every .5 sec
	{
		m_averageDataRateList.push_front(g_eMuleApp.stat_sessionSentBytes);
		m_averageTickList.push_front(curTick);

		// remove anything older than 40  seconds
		while(curTick-m_averageTickList.back() > 40000)
		{
   			m_averageDataRateList.pop_back();
			m_averageTickList.pop_back();
		}
	}
    // Save time to be able to calculate how long between each call
    m_lastGaveDataTick = curTick;

	// LANCAST (moosetea) - If we have more than one LAN user were going to upload to them
	// Were going to try uploading the way plus way....... 
	if (QtyLANClients > 0){

        bool clientWasReady = false;

	    POSITION pos = uploadinglist.GetHeadPosition();
	    while (pos != NULL){
            // Get the client. Note! Also updates activeClientsListPos as a side effect.
		    CUpDownClient* cur_client = uploadinglist.GetNext(pos);
            bool tempWantsMoreBandwidth = false;

			// LANCAST (moosetea) - Just Count LAN Clients
			if (cur_client->IsOnLAN())
				cur_client->SendBlockData(m_guessedMaxLANBandwidth, tempWantsMoreBandwidth);

            if(tempWantsMoreBandwidth == true)
                clientWasReady = true;
      
	    }

	    if (clientWasReady == false){
            if(m_guessedMaxLANBandwidth > 3000) {
		        m_guessedMaxLANBandwidth -= 2000;
            } else {
               m_guessedMaxLANBandwidth = 1000;
            }
	    } else {
		    m_guessedMaxLANBandwidth += 2000;
	    }

	}

	// START enkeyDEV(th1) -L2HAC- lowid side, automatic callback phase
#ifdef OLD_SOCKETS_ENABLED
	if (g_eMuleApp.m_pServerConnect->IsConnected() && g_eMuleApp.m_pServerConnect->IsLowID())
	{
		for (POSITION pos = waitinglist.GetHeadPosition();pos != 0;waitinglist.GetNext(pos))
		{
			CUpDownClient* cur_client = waitinglist.GetAt(pos);
			if (cur_client->IsL2HACEnabled()
				&& cur_client->GetLastL2HACExecution()
				&& cur_client->GetL2HACTime()
				&& (curTick - cur_client->GetLastL2HACExecution()) > cur_client->GetL2HACTime())
			{
				if (g_eMuleApp.m_pListenSocket->TooManySockets() && !(cur_client->m_pRequestSocket && cur_client->m_pRequestSocket->IsConnected()) )
					cur_client->SetLastL2HACExecution(curTick - cur_client->GetL2HACTime() + static_cast<uint32>(ROUND((static_cast<double>(rand())/RAND_MAX)*300000.0)));
				else
				{
					cur_client->DisableL2HAC();
					if (!cur_client->HasLowID() && cur_client->GetL2HACTime())
					cur_client->TryToConnect();
				}
			}
		}
	}
#endif //OLD_SOCKETS_ENABLED
	// END enkeyDEV(th1) -L2HAC-

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUploadQueue::CanAcceptNewClient(uint32 numberOfUploads)
{
	EMULE_TRY

//	Check if we can allow a new client to start downloading from us
	if (numberOfUploads < MIN_UP_CLIENTS_ALLOWED)
		return true;
	else if (numberOfUploads >= MAX_UP_CLIENTS_ALLOWED)
		return false;

//	Now the final check
	if (g_eMuleApp.m_pGlobPrefs->GetMaxUpload() == UNLIMITED)
	{
		if (numberOfUploads < (GetDataRate()/UPLOAD_CLIENT_DATARATE))
			return true;
	}
	else
	{
		if (numberOfUploads < (uint32)(g_eMuleApp.m_pGlobPrefs->GetMaxUpload()*1024/UPLOAD_LOW_CLIENT_DR)+1)
			return true;
	}
//	Nope
	return false;

	EMULE_CATCH

//	Nope
	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUploadQueue::~CUploadQueue(){
	KillTimer(0, m_hTimer);
	if ( m_hSaveSTimer )
	{
		m_pSaveThread->SuspendThread();
		__int64         qwDueTime;
		LARGE_INTEGER   liDueTime;
		qwDueTime = -1;         // Copy the relative time into a LARGE_INTEGER.
		liDueTime.LowPart  = (DWORD) ( qwDueTime & 0xFFFFFFFF );
		liDueTime.HighPart = (LONG)  ( qwDueTime >> 32 );
		// re-Set the timer to me inworked each second
		::SetWaitableTimer(m_hSaveSTimer,&liDueTime,1,NULL,NULL,TRUE);
		// Set the thread to Get more CPU to finish fast
		m_pSaveThread->SetThreadPriority(THREAD_PRIORITY_HIGHEST+ g_eMuleApp.m_pGlobPrefs->GetMainProcessPriority());
		HANDLE hArr[2];
		hArr[0]	=	m_hSaveSTimer;
		hArr[1]	=	m_pSaveThread->m_hThread;
		// set thread to exit 
		m_bExiting	=	TRUE;
		// Resume the thread
		m_pSaveThread->ResumeThread();
		// Wait until thread finishes 10000 sec - this should be enough
		if ( WaitForMultipleObjects(2,hArr,TRUE,10000000) == WAIT_TIMEOUT )
		{
			TRACE("Thread known.met thread did not exited on time");
		}
	}
}

POSITION CUploadQueue::GetWaitingClient(CUpDownClient* client)
{
	EMULE_TRY

	return waitinglist.Find(client); 

	EMULE_CATCH

	return 0;
}

/*POSITION CUploadQueue::GetWaitingClientByID(CUpDownClient* client)
{
	EMULE_TRY

	for (POSITION pos = waitinglist.GetHeadPosition();pos != 0;waitinglist.GetNext(pos)){
		if (client->Compare(waitinglist.GetAt(pos)))
			return pos;
	}

	EMULE_CATCH

	return 0;
}*/

CUpDownClient* CUploadQueue::GetWaitingClientByIP_UDP(uint32 dwIP, uint16 dwport)
{
	EMULE_TRY

	//v- eklmn: optimization: don't call waitinglist.GetNext(pos) many times
	for (POSITION pos = waitinglist.GetHeadPosition();pos != 0;) {
		CUpDownClient* cur_client = waitinglist.GetNext(pos);
		if (dwIP == cur_client->GetIP())
			return cur_client;
        }

	EMULE_CATCH

	return 0;
}

/*
POSITION CUploadQueue::GetDownloadingClient(CUpDownClient* client)
{
	EMULE_TRY

	for (POSITION pos = uploadinglist.GetHeadPosition();pos != 0;uploadinglist.GetNext(pos)){
		if (client == uploadinglist.GetAt(pos))
			return pos;
	}

	EMULE_CATCH

	return 0;
}
*/
uint32	CUploadQueue::GetDataRateShown()		//Cax2 -  roughly 5 seconds sample rate: more immediate data rate feedback.
{
	if(m_averageDataRateList.size() >9)		//Cax2 - do we have those '5 seconds'? (can be messed up by program freezes, etc)
	{
		list<uint64>::iterator dr=m_averageDataRateList.begin();
		list<DWORD>::iterator tick=m_averageTickList.begin();
		advance(dr,9);
		advance(tick,9);
		return (uint32)( (m_averageDataRateList.front() - (*dr))*1000.0f/(m_averageTickList.front() - (*tick)) );
	}
	else
		return GetDataRate();
}

uint32	CUploadQueue::GetDataRate()
{
    // Calculate average data rate
    if(GetTickCount()-m_lastCalculatedDataRateTick > 1000) {
        if(m_averageDataRateList.size() >1) {
	        m_iDataRate = static_cast<uint32>(static_cast<double>(m_averageDataRateList.front()-m_averageDataRateList.back())*1000.0 / (m_averageTickList.front()-m_averageTickList.back()));
        } else {
            m_iDataRate = 0;
        }
    }

    return m_iDataRate;
}


void CUploadQueue::UpdateBanCount()
{
	EMULE_TRY

	int count=0;
	for (POSITION pos = waitinglist.GetHeadPosition();pos != 0;waitinglist.GetNext(pos)){
		CUpDownClient* cur_client= waitinglist.GetAt(pos);
		if(cur_client->IsBanned())
			count++;
	}
	SetBanCount(count);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CUploadQueue::AddClientToQueue(CUpDownClient* client, bool bIgnoreTimelimit)
{
	EMULE_TRY

#ifdef OLD_SOCKETS_ENABLED
//	If we're connected to the server, and we're LOWID, and the client is on a remote server and is not downloading
//		and is not a friend and the queue is longer than 50...
	if ( g_eMuleApp.m_pServerConnect->IsConnected()
	  && g_eMuleApp.m_pServerConnect->IsLowID()
		&& !g_eMuleApp.m_pServerConnect->IsLocalServer(client->GetServerIP(),client->GetServerPort())
		&& client->GetDownloadState() == DS_NONE 
		&& !client->IsFriend()
		&& GetWaitingUserCount() > 50)
		return;
#endif //OLD_SOCKETS_ENABLED

	//eklmn: check if file was requested with OP_SETREQFILEID, OP_FILEREQUEST or OP_STARTUPLOADREQ
	CKnownFile* m_pReqPartFile = g_eMuleApp.m_pSharedFilesList->GetFileByID((uchar*)client->m_reqFileHash);

	// if file wasn't requested
	if (!m_pReqPartFile)
		return;

	//(filtering invalid eMule clients)
	if(client->GetMuleVersion()==0 && client->GetVersion()==0 && (client->GetClientSoft()==SO_EMULE || client->GetClientSoft()==SO_OLDEMULE)) {
		return;
	}
	//end (filtering invalid eMule clients) // Mr Faber / netwolf

	client->AddAskedCount();
	client->SetLastUpRequest();

	// START enkeyDEV(th1) -L2HAC- lowid side
	client->SetLastL2HACExecution();
	if (!client->HasLowID() && client->GetL2HACTime()) client->EnableL2HAC();
	// END enkeyDEV(th1) -L2HAC-

	// LANCAST (moosetea) - We ignore any ban LAN clients may have :)
	if (!bIgnoreTimelimit && !client->IsOnLAN()){
		if (client->IsBanned()){
			if (GetTickCount() - client->GetBanTime() > g_eMuleApp.m_pGlobPrefs->BadClientBanTime()){
				client->UnBan();
			}
			else
				return;
		}
		client->AddRequestCount(client->m_reqFileHash);
	}
	// check for double
	for (POSITION pos = waitinglist.GetHeadPosition();pos != 0;waitinglist.GetNext(pos)){
		CUpDownClient* cur_client= waitinglist.GetAt(pos);
		if (cur_client == client){	//already on queue
			client->SendRankingInfo();
			g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlQueueList.UpdateClient(client);
			return;			
		}
/* MERGE26
		// changing back, to avoid hack/mldonkey problems
		else if (client->GetIP() == cur_client->GetIP() || !memcmp(cur_client->GetUserHash(),client->GetUserHash(),16) ) {
		//else if ((client->GetIP() == cur_client->GetIP() && client->GetUserPort() == cur_client->GetUserPort())|| !memcmp(cur_client->GetUserHash(),client->GetUserHash(),16)){ // Lord KiRon
*/
		else if ( client->Compare(cur_client) ) {
			// another client with same ip or hash
			if (g_eMuleApp.m_pGlobPrefs->IsSecureIdentEnabled())
			{
			
				if (client->m_pCredits->GetCurrentIdentState(client->GetIP()) == IS_IDENTIFIED && cur_client->m_pCredits->GetCurrentIdentState(cur_client->GetIP()) != IS_IDENTIFIED)
			
				{
					AddDebugLogLine(false, RGB_LOG_DIMMED + GetResString(IDS_SAMEUSERHASH),client->GetUserName(),cur_client->GetUserName(),cur_client->GetUserName() );
					RemoveFromWaitingQueue(pos,true);
				} else
				if (client->m_pCredits->GetCurrentIdentState(client->GetIP()) != IS_IDENTIFIED && cur_client->m_pCredits->GetCurrentIdentState(cur_client->GetIP()) == IS_IDENTIFIED)
				{
					AddDebugLogLine(false, RGB_LOG_DIMMED + GetResString(IDS_SAMEUSERHASH),client->GetUserName(),cur_client->GetUserName(),client->GetUserName() );		
					return;
				} else
				if (client->m_pCredits->GetCurrentIdentState(client->GetIP()) != IS_IDENTIFIED && cur_client->m_pCredits->GetCurrentIdentState(cur_client->GetIP()) != IS_IDENTIFIED)
				{
					AddDebugLogLine(false, RGB_LOG_DIMMED + GetResString(IDS_SAMEUSERHASH),client->GetUserName(),cur_client->GetUserName(),"both" );
					RemoveFromWaitingQueue(pos,true);
					return;
				} else
			// EC - Can this happen? Maybe client tries to re-enter queue
				if (client->m_pCredits->GetCurrentIdentState(client->GetIP()) == IS_IDENTIFIED && cur_client->m_pCredits->GetCurrentIdentState(cur_client->GetIP()) == IS_IDENTIFIED)
				{
					AddDebugLogLine(false, RGB_LOG_DIMMED + _T("-->Two Identified clients (%s) and (%s)  have the same Userhash or IP, Removing %s"),client->GetUserName(),cur_client->GetUserName(),cur_client->GetUserName());
					RemoveFromWaitingQueue(pos,true);	
				} 
			} else 
			{
				AddDebugLogLine(false, RGB_LOG_DIMMED + GetResString(IDS_SAMEUSERHASH),client->GetUserName(),cur_client->GetUserName(),cur_client->GetUserName() );
				RemoveFromWaitingQueue(pos,true);	
			}
			
#ifdef OLD_SOCKETS_ENABLED
			if (cur_client->m_pRequestSocket == NULL)
			{
				if (cur_client->Disconnected())
					cur_client = NULL;
			}
#endif //OLD_SOCKETS_ENABLED
			return;
		}
	}
	// done

/*
	if(ASK_FOR_PARTS_ON_UPLOAD)
		client->SendFileRequestForUpload();
*/

	// Add clients server to list.
	if (g_eMuleApp.m_pGlobPrefs->GetAddServersFromConn() && client->GetServerIP() && client->GetServerPort()){	//SyruS (0.29c)
		in_addr host;
		host.S_un.S_addr = client->GetServerIP();
		CString s(inet_ntoa(host));
		CServer* srv = new CServer(client->GetServerPort(), s);
		srv->SetListName(srv->GetAddress());
		if (!g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.AddServer(srv, true))
			delete srv;
		/*else
			AddLogLine(false, _T("Added new server: %s:%d"), srv->GetFullIP(), srv->GetPort());*/
	}

	// statistic values
	//CKnownFile* m_pReqPartFile = g_eMuleApp.m_pSharedFilesList->GetFileByID((uchar*)client->m_reqFileHash);
	if (m_pReqPartFile) m_pReqPartFile->statistic.AddRequest();
	
	// TODO find better ways to cap the list
	if ((uint32)waitinglist.GetCount() >= (g_eMuleApp.m_pGlobPrefs->GetQueueSize()+m_iBannedCount))
		return;
	if (client->IsDownloading()){
		// he's already downloading and wants probably only another file
		Packet* packet = new Packet(OP_ACCEPTUPLOADREQ);
		g_eMuleApp.m_pUploadQueue->AddUpDataOverheadFileRequest(packet->m_dwSize);
#ifdef OLD_SOCKETS_ENABLED
		client->m_pRequestSocket->SendPacket(packet,true);
#endif //OLD_SOCKETS_ENABLED
		client->SetLastGotULData();	//eklmn: set Upload Timer
		return;
	}
	//if (waitinglist.IsEmpty() && CanAcceptNewClient())
	//	AddUpNextClient(client);
	//else{

	// LANCAST (moosetea) - Lan users bypass the queue
	if (client->IsOnLAN())
		AddUpNextClient(client);
	else{
		waitinglist.AddTail(client);
		client->SetUploadState(US_ONUPLOADQUEUE);
		client->SendRankingInfo();
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlQueueList.AddClient(client);
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateUploadHeader();
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUploadQueue::RemoveFromUploadQueue(CUpDownClient* client, EnumEndTransferSession eReason, bool updatewindow )
{
	if (client == NULL)
		return false;
	
	EMULE_TRY

	POSITION removed_pos = 0;

	removed_pos = uploadinglist.Find(client);
	if (removed_pos)
	{
		uploadinglist.RemoveAt(removed_pos);
		
		if (updatewindow)
			g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlUploadList.RemoveClient(client);

		//eklmn: Update a a stats
		//           if client got some byte s from us the session will succesfull, otherwise failed
		if (client->GetTransferredUp())
		{
			m_iULSessionSuccessful++;
			if (client->GetTransferredUp() >= PARTSIZE)
				m_iULSessionSuccessfulFullChunk++;
			else
				m_iULSessionSuccessfulPartChunk[eReason]++;
		}
		else
		{
			m_iULSessionFailed[eReason]++;
		}
		
		// eklmn: if we transfer something we need to update a upload time
		if (client->GetTransferredUp())
			m_iTotalUploadTime += client->GetUpStartTimeDelay()/1000;

		client->SetUploadState(US_NONE);
		client->ClearUploadBlockRequests();

//		//report the PartFile that UL is finished
//		if (client->pUploadingPartFile)
//		{
//			client->pUploadingPartFile->SetFileUpload(false);
//			client->pUploadingPartFile = NULL;
//		}

		// Added by Tarod -saves known.met file to avoid loosing it by accident
		g_eMuleApp.m_pKnownFilesList->Save();
		return true;
	}

	EMULE_CATCH

	return false;
}

uint32 CUploadQueue::GetAverageUpTime()
{
	if( m_iULSessionSuccessful )
	{
		return m_iTotalUploadTime/m_iULSessionSuccessful;
	}
	return 0;
}

bool CUploadQueue::RemoveFromWaitingQueue(CUpDownClient* client, bool updatewindow)
{
	EMULE_TRY

	POSITION pos = waitinglist.Find(client);
	if (pos){
		RemoveFromWaitingQueue(pos,updatewindow);
		if (updatewindow)
			g_eMuleApp.m_pdlgEmule->m_wndTransfer.UpdateUploadHeader(); // changed by FoRcHa
		return true;
	}

	EMULE_CATCH

	return false;
}

void CUploadQueue::RemoveFromWaitingQueue(POSITION pos, bool updatewindow)
{
	EMULE_TRY

	CUpDownClient* todelete = waitinglist.GetAt(pos);
	waitinglist.RemoveAt(pos);
	if(todelete == NULL)
		return;
	if( todelete->IsBanned() )
	{
#if KEEP_BANNED_CLIENTS
		//SyruS don't unban and remove from queue
		if (!g_eMuleApp.m_pGlobPrefs->GetCMNotLog())
			AddDebugLogLine(false, RGB_LOG_DIMMED + _T("(NOT YET) Keeping banned client '%s' in waitingqueue"), todelete->GetUserName());
		//1st idea (dont use yet)... return;
#endif
		todelete->UnBan();
	}

	if (updatewindow)
		g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlQueueList.RemoveClient(todelete);
	todelete->SetUploadState(US_NONE);

	EMULE_CATCH
}



bool CUploadQueue::CheckForTimeOver(CUpDownClient* client)
{
	EMULE_TRY

	uint32 score = client->GetScore(true,true); //Maella - Less CPU usage
	for (POSITION pos = waitinglist.GetHeadPosition();pos != 0;waitinglist.GetNext(pos)){
		if (score < waitinglist.GetAt(pos)->GetScore(true,false))
			return true;
	}

	EMULE_CATCH

	return false;
}


void CUploadQueue::DeleteAll()
{
	EMULE_TRY

	waitinglist.RemoveAll();
	uploadinglist.RemoveAll();

	EMULE_CATCH
}

uint16 CUploadQueue::GetWaitingPosition(CUpDownClient* client)
{
	EMULE_TRY

	if (!IsOnUploadQueue(client))
		return 0;
	uint16 rank = 1;
	uint32 myscore = client->GetScore(false);
	for (POSITION pos = waitinglist.GetHeadPosition();pos != 0;waitinglist.GetNext(pos)){
		if (waitinglist.GetAt(pos)->GetScore(false) > myscore)
			rank++;
	}
	return rank;

	EMULE_CATCH

	return 0;
}

void CUploadQueue::CompUpDataRateOverhead()
{
	EMULE_TRY

	m_AvarageUDRO_list.AddTail(m_nUpDataRateMSOverhead);
	if (m_AvarageUDRO_list.GetCount() > 150)
		m_AvarageUDRO_list.RemoveAt(m_AvarageUDRO_list.GetHeadPosition());
	m_nUpDataRateOverhead = 0;
	m_nUpDataRateMSOverhead = 0;
	for (POSITION pos = m_AvarageUDRO_list.GetHeadPosition();pos != 0;m_AvarageUDRO_list.GetNext(pos))
		m_nUpDataRateOverhead += m_AvarageUDRO_list.GetAt(pos);

	if(m_AvarageUDRO_list.GetCount() > 10)
		m_nUpDataRateOverhead = 10*m_nUpDataRateOverhead/m_AvarageUDRO_list.GetCount();
	else
		m_nUpDataRateOverhead = 0;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
VOID CALLBACK TimerProc(HWND hwnd, UINT uMsg,UINT_PTR idEvent,DWORD dwTime)
{
	EMULE_TRY

	// Barry - Don't do anything if the app is shutting down - can cause unhandled exceptions
	if (!g_eMuleApp.m_pdlgEmule->IsRunning())
		return;

	g_eMuleApp.m_pClientList->ResetAllValueCaches();
	g_eMuleApp.m_pUploadQueue->Process();
	g_eMuleApp.m_pDownloadQueue->Process();

	//--- xrmb:statsautosave ---
	//--- it might be not the best idea to do this here :) ---
	//g_eMuleApp.m_pKnownFilesList->Process();		//Cax2 moved back to SaveSourcesThread() - this function is deprecated! :P
	//--- :xrmb ---
	
    g_eMuleApp.m_pUploadQueue->CompUpDataRateOverhead();
	g_eMuleApp.m_pDownloadQueue->CompDownDataRateOverhead();
	g_iCounter++;

	// one second
	if (g_iCounter >= 10)
	{
		g_iCounter=0;
	        g_eMuleApp.m_pClientCreditList->Process();
		g_eMuleApp.m_pServerList->Process();
		g_eMuleApp.m_pFriendList->Process();
#ifdef OLD_SOCKETS_ENABLED
		if( g_eMuleApp.m_pServerConnect->IsConnecting())
			g_eMuleApp.m_pServerConnect->TryAnotherConnectionRequest();
#endif //OLD_SOCKETS_ENABLED

		g_eMuleApp.m_pdlgEmule->m_dlgStatistics.UpdateConnectionsStatus();

#ifdef OLD_SOCKETS_ENABLED
		if (g_eMuleApp.m_pServerConnect->IsConnecting()) g_eMuleApp.m_pServerConnect->CheckForTimeout();
#endif //OLD_SOCKETS_ENABLED

		// -khaos--+++> Update connection stats...
		g_iUpdateConnStats++;
		// 2 seconds
		if (g_iUpdateConnStats>=2) 
		{
			g_iUpdateConnStats=0;
			g_eMuleApp.m_pdlgEmule->m_dlgStatistics.UpdateConnectionStats((double)g_eMuleApp.m_pUploadQueue->GetDataRate()/1024, static_cast<double>(g_eMuleApp.m_pDownloadQueue->GetDataRate())/1024.0);
		}
		// <-----khaos-

		// display graphs
		if(g_eMuleApp.m_pGlobPrefs->GetTrafficOMeterInterval()>0) 
		{
			g_iGraph++;

			if (g_iGraph >= (uint32)(g_eMuleApp.m_pGlobPrefs->GetTrafficOMeterInterval()) ) 
			{
				g_iGraph=0;
				g_eMuleApp.m_pdlgEmule->m_dlgStatistics.SetCurrentRate((static_cast<double>(g_eMuleApp.m_pUploadQueue->GetDataRateShown()))/1024.0,((double)g_eMuleApp.m_pDownloadQueue->GetDataRate())/1024);
			}
		}
		if (g_eMuleApp.m_pdlgEmule->m_pdlgActive == &g_eMuleApp.m_pdlgEmule->m_dlgStatistics && g_eMuleApp.m_pdlgEmule->IsWindowVisible() )  
		{
			// display stats
			if (g_eMuleApp.m_pGlobPrefs->GetStatsInterval()>0) 
			{
				g_iStats++;

				if (g_iStats >= (uint32)(g_eMuleApp.m_pGlobPrefs->GetStatsInterval()) ) 
				{
					g_iStats=0;
					g_eMuleApp.m_pdlgEmule->ShowStatistics();
				}
			}
		}
		//save rates every second
		g_eMuleApp.m_pdlgEmule->m_dlgStatistics.RecordRate();
		// mobilemule sockets
#ifdef OLD_SOCKETS_ENABLED
#ifdef MOBILE_MULE
		g_eMuleApp.m_pMMServer->Process();
#endif //MOBILE_MULE
#endif //OLD_SOCKETS_ENABLED

		g_iSec++;
		// 5 seconds
		if (g_iSec>=5) 
		{
			#ifdef _DEBUG
				if (!AfxCheckMemory())
				{
					g_eMuleApp.AddLogLine(false, RGB_PINK + _T("** ") + RGB_LOG_ERROR + _T("Memory corruption detected") + RGB_PINK + _T(" **"));
//					AfxDebugBreak();
				}
			#endif

			g_iSec = 0;
		#ifdef OLD_SOCKETS_ENABLED
			g_eMuleApp.m_pListenSocket->Process();
		#endif //OLD_SOCKETS_ENABLED
			g_eMuleApp.OnlineSig(); // Added By Bouc7 
			g_eMuleApp.m_pdlgEmule->ShowTransferRate();	// eklmn: bugfix(11): crazy output in statusbar (update only tray or window)
		//	TODO - Cax2 - need to refresh 'stuck' clients
		//	if (g_eMuleApp.m_pdlgEmule->m_pdlgActive == &g_eMuleApp.m_pdlgEmule->m_dlgStatistics)
		//		g_eMuleApp.m_pdlgEmule->ShowStatistics();
		}

		g_iStatSave++;
		if (g_iStatSave>=60) 
		{
			// Time to save our cumulative statistics.
			g_iStatSave=0;
			
			// -khaos--+++>
			// CPreferences::SaveStats() in Preferences.cpp
			// This function does NOT update the tree!
			g_eMuleApp.m_pGlobPrefs->SaveStats();
			// <-----khaos-

			g_eMuleApp.m_pServerConnect->KeepConnectionAlive();

			// obaldin/DonGato - Scheduler check
			if(g_eMuleApp.m_pGlobPrefs->IsSCHEnabled())
			{
				CTime curr_t = CTime::GetCurrentTime();
				uint32 secs = curr_t.GetSecond() + 60*curr_t.GetMinute() + 60*60*curr_t.GetHour();

				if(secs > g_eMuleApp.m_pGlobPrefs->GetSCHShift1()-120 && secs < g_eMuleApp.m_pGlobPrefs->GetSCHShift1()+120)
				{
					// switching to shift1 speeds
					g_eMuleApp.m_pUploadQueue->SCHShift1UploadCheck();
					g_eMuleApp.m_pGlobPrefs->SetMaxUpload(g_eMuleApp.m_pGlobPrefs->GetSCHShift1Upload());
					g_eMuleApp.m_pGlobPrefs->SetMaxDownload(g_eMuleApp.m_pGlobPrefs->GetSCHShift1Download());
					g_eMuleApp.m_pGlobPrefs->SetMaxConnections(g_eMuleApp.m_pGlobPrefs->GetSCHShift1conn());
					g_eMuleApp.m_pGlobPrefs->SetMaxDownloadConperFive(g_eMuleApp.m_pGlobPrefs->GetSCHShift15sec());
					// Purity - Sending message for scheduler shift 1
					CString MessageText;
					MessageText.Format(CString("Switching to Shift 1"));
					g_eMuleApp.m_pSMTPConnection->SendMail(MessageText, g_eMuleApp.m_pGlobPrefs->GetUseSchedulerNotifier(), g_eMuleApp.m_pGlobPrefs->IsSMTPInfoEnabled());
					g_eMuleApp.m_pdlgEmule->ShowNotifier(MessageText, TBN_SCHEDULER, false, g_eMuleApp.m_pGlobPrefs->GetUseSchedulerNotifier());
					CLoggable::AddDebugLogLine(true, CString("Switching to Shift 1"));
				} 
				else if (secs > g_eMuleApp.m_pGlobPrefs->GetSCHShift2()-120 && secs < g_eMuleApp.m_pGlobPrefs->GetSCHShift2()+120) 
				{
                    			int dayOfWeek = curr_t.GetDayOfWeek();
					if ((dayOfWeek==2 && g_eMuleApp.m_pGlobPrefs->IsSCHExceptMon()) 
						|| (dayOfWeek==3 && g_eMuleApp.m_pGlobPrefs->IsSCHExceptTue()) 
						|| (dayOfWeek==4 && g_eMuleApp.m_pGlobPrefs->IsSCHExceptWed()) 
						|| (dayOfWeek==5 && g_eMuleApp.m_pGlobPrefs->IsSCHExceptThu()) 
						|| (dayOfWeek==6 && g_eMuleApp.m_pGlobPrefs->IsSCHExceptFri()) 
						|| (dayOfWeek==7 && g_eMuleApp.m_pGlobPrefs->IsSCHExceptSat()) 
						|| (dayOfWeek==1 && g_eMuleApp.m_pGlobPrefs->IsSCHExceptSun()))
                    			{
                        			CLoggable::AddDebugLogLine(true, CString("Day excepted!"));
					} 
					else 
					{
						// switching to shift2 speeds
						g_eMuleApp.m_pUploadQueue->SCHShift2UploadCheck();
						g_eMuleApp.m_pGlobPrefs->SetMaxUpload(g_eMuleApp.m_pGlobPrefs->GetSCHShift2Upload());
						g_eMuleApp.m_pGlobPrefs->SetMaxDownload(g_eMuleApp.m_pGlobPrefs->GetSCHShift2Download());
						g_eMuleApp.m_pGlobPrefs->SetMaxConnections(g_eMuleApp.m_pGlobPrefs->GetSCHShift2conn());
						g_eMuleApp.m_pGlobPrefs->SetMaxDownloadConperFive(g_eMuleApp.m_pGlobPrefs->GetSCHShift25sec());
						// Purity - Sending message for scheduler shift 2
						CString MessageText;
						MessageText.Format(CString("Switching to Shift 2"));
						g_eMuleApp.m_pSMTPConnection->SendMail(MessageText, g_eMuleApp.m_pGlobPrefs->GetUseSchedulerNotifier(), g_eMuleApp.m_pGlobPrefs->IsSMTPInfoEnabled());
						g_eMuleApp.m_pdlgEmule->ShowNotifier(MessageText, TBN_SCHEDULER, false, g_eMuleApp.m_pGlobPrefs->GetUseSchedulerNotifier());
						CLoggable::AddDebugLogLine(true, CString("Switching to Shift 2"));
					}
				}
			}
		}
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CUpDownClient* CUploadQueue::GetNextClient(CUpDownClient* lastclient)
{
	EMULE_TRY

	if (waitinglist.IsEmpty())
		return NULL;
	if (!lastclient)
		return waitinglist.GetHead();
	POSITION pos = waitinglist.Find(lastclient);
	if (!pos){
		TRACE("Error: CServerList::GetNextClient");
		return waitinglist.GetHead();
	}
	waitinglist.GetNext(pos);
	if (!pos)
		return NULL;
	else
		return waitinglist.GetAt(pos);

	EMULE_CATCH

	return NULL;
}

void CUploadQueue::FindSourcesForFileById(CTypedPtrList<CPtrList, CUpDownClient*>* srclist, const uchar* filehash) 
{
	EMULE_TRY

	POSITION pos;
	
	pos = uploadinglist.GetHeadPosition();
	while(pos) {
		CUpDownClient *potential = uploadinglist.GetNext(pos);
		if(md4cmp(potential->m_reqFileHash, filehash) == 0)
			srclist->AddTail(potential);
	}

	pos = waitinglist.GetHeadPosition();
	while(pos) {
		CUpDownClient *potential = waitinglist.GetNext(pos);
		if(md4cmp(potential->m_reqFileHash, filehash) == 0)
			srclist->AddTail(potential);
	}

	EMULE_CATCH
}


UINT SaveSourcesThread(HANDLE lpArg)
{
	HANDLE hTimer;
	hTimer	=	((CUploadQueue*)lpArg)->m_hSaveSTimer;
	if ( !hTimer )
	{
		return -1;
	}
    __int64         qwDueTime;
     LARGE_INTEGER   liDueTime;
    qwDueTime = -30*60*1000;         // Copy the relative time into a LARGE_INTEGER.
    liDueTime.LowPart  = (DWORD) ( qwDueTime & 0xFFFFFFFF );
    liDueTime.HighPart = (LONG)  ( qwDueTime >> 32 );
	// Set timer to signal each 30 min 
	::SetWaitableTimer(hTimer,&liDueTime,30*60*1000,NULL,NULL,TRUE);
	bool bFirstRun	=	true;
	while (!(((CUploadQueue*)lpArg)->m_bExiting) )
	{
		// Wait for timer signal 
		::WaitForSingleObject(hTimer,INFINITE);
		// If eMule supposed to exit
		if ( (((CUploadQueue*)lpArg)->m_bExiting) )
		{
			// break and finish the thread
			break;
		}
		// avoid running right away
		if ( bFirstRun )
		{
			bFirstRun	=	false;
			continue;
		}
		SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_BELOW_NORMAL+ g_eMuleApp.m_pGlobPrefs->GetMainProcessPriority());
		// process (save) sources each 30 min
		//g_eMuleApp.m_pKnownFilesList->Process();			//Cax2 - no more need for that..
		g_eMuleApp.m_pKnownFilesList->Save();	
		SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_NORMAL+ g_eMuleApp.m_pGlobPrefs->GetMainProcessPriority());
	}
	// Close timer also to signal that we finished
	::CloseHandle(hTimer);
	return 0;
}

void CUploadQueue::SCHShift1UploadCheck()
{
	EMULE_TRY

	if(g_eMuleApp.m_pGlobPrefs->GetSCHShift1Upload() != 0)
	{
		if(g_eMuleApp.m_pGlobPrefs->GetSCHShift1Upload() < 4 && (g_eMuleApp.m_pGlobPrefs->GetSCHShift1Upload()*3 < g_eMuleApp.m_pGlobPrefs->GetSCHShift1Download()))
			g_eMuleApp.m_pGlobPrefs->SetSCHShift1Download(g_eMuleApp.m_pGlobPrefs->GetSCHShift1Upload()*3);

		if(g_eMuleApp.m_pGlobPrefs->GetSCHShift1Upload() < 10 && (g_eMuleApp.m_pGlobPrefs->GetSCHShift1Upload()*4 < g_eMuleApp.m_pGlobPrefs->GetSCHShift1Download()))
			g_eMuleApp.m_pGlobPrefs->SetSCHShift1Download(g_eMuleApp.m_pGlobPrefs->GetSCHShift1Upload()*4);
	}

	EMULE_CATCH
}

void CUploadQueue::SCHShift2UploadCheck()
{
	EMULE_TRY

	if(g_eMuleApp.m_pGlobPrefs->GetSCHShift2Upload() != 0)
	{
		if(g_eMuleApp.m_pGlobPrefs->GetSCHShift2Upload() < 4 && (g_eMuleApp.m_pGlobPrefs->GetSCHShift2Upload()*3 < g_eMuleApp.m_pGlobPrefs->GetSCHShift2Download()))
			g_eMuleApp.m_pGlobPrefs->SetSCHShift2Download(g_eMuleApp.m_pGlobPrefs->GetSCHShift2Upload()*3);

		if(g_eMuleApp.m_pGlobPrefs->GetSCHShift2Upload() < 10 && (g_eMuleApp.m_pGlobPrefs->GetSCHShift2Upload()*4 < g_eMuleApp.m_pGlobPrefs->GetSCHShift2Download()))
			g_eMuleApp.m_pGlobPrefs->SetSCHShift2Download(g_eMuleApp.m_pGlobPrefs->GetSCHShift2Upload()*4);
	}

	EMULE_CATCH
}

bool CUploadQueue::IsFileUploading (CPartFile* check_file)
{
	EMULE_TRY

	CUpDownClient* cur_client;
	for(POSITION pos = uploadinglist.GetHeadPosition(); pos != NULL; uploadinglist.GetNext(pos))
	{
		cur_client = uploadinglist.GetAt(pos);
		if (cur_client->m_pReqPartFile == check_file)
			return true;
	}
		
	return false;
	EMULE_CATCH
	return true;
}
