//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "emule.h"
#include "emuleDlg.h"
#include "opcodes.h"
#include "WebServer.h"
#include "KeyboardShortcut.h"

#if JUMPSTART
#include "Jumpstarter.h"
#endif // JUMPSTART

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW

#endif

// CEmuleApp

BEGIN_MESSAGE_MAP(CEmuleApp, CWinApp)
	ON_COMMAND(ID_HELP, OnHelp)
END_MESSAGE_MAP()

CEmuleApp				g_eMuleApp;
const static UINT		UWM_ARE_YOU_EMULE = RegisterWindowMessage(_T(EMULE_GUID));
#ifdef _DEBUG
static CMemoryState		oldMemState, newMemState, diffMemState;
#endif _DEBUG

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CEmuleApp::CEmuleApp()
{
	m_pDump = NULL;
	m_qwComCtrlVer = MAKEDLLVERULL(4,0,0,0);
	m_hSystemImageList = NULL;
	m_sizSmallSystemIcon.cx = GetSystemMetrics(SM_CXSMICON);
	m_sizSmallSystemIcon.cy = GetSystemMetrics(SM_CYSMICON);

//	Get session start time offset
	stat_starttime = ::GetTickCount();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CEmuleApp Initialising
BOOL CEmuleApp::InitInstance()
{
	EMULE_TRY
#ifdef _DEBUG
	oldMemState.Checkpoint();
#endif

//	We have to catch exception by ourselves!
	m_pDump = new MiniDumper(CString("eMule Plus ") + CURRENT_VERSION_LONG);

//#ifdef OLD_SOCKETS_ENABLED
//	We need sockets started before we proccess the preferences
	if (!AfxSocketInit())
	{		
		AfxMessageBox(IDS_SOCKETS_INIT_FAILED);
		return FALSE;
	}
//#endif //OLD_SOCKETS_ENABLED

//	Create & initialize all the important stuff 
	m_pstrPendingLink = 0;
	bool bIsRunningButNoCmd = false;

	int iResult = ProcessCommandline();

	if (iResult == 0)
		return false;
	else if (iResult == 1) 
		bIsRunningButNoCmd = true;

	m_pGlobPrefs = new CPreferences();
	
	//EpicStruggle
//	If we don't want to allow multiple instances...
	if(bIsRunningButNoCmd && !g_eMuleApp.m_pGlobPrefs->GetMultiple())
		return false;

//	InitCommonControls() is required on Windows XP if an application
//	manifest specifies use of ComCtl32.dll version 6 or later to enable
//	visual styles.  Otherwise, any window creation will fail.
	InitCommonControls();
	DWORD dwComCtrlMjr = 4;
	DWORD dwComCtrlMin = 0;
	AtlGetCommCtrlVersion(&dwComCtrlMjr, &dwComCtrlMin);
	m_qwComCtrlVer = MAKEDLLVERULL(dwComCtrlMjr,dwComCtrlMin,0,0);

	CWinApp::InitInstance();

//	Enable containment of OLE controls
	AfxEnableControlContainer();

//	Set the priority of the processes main thread
	SetThreadPriority(THREAD_PRIORITY_NORMAL+ m_pGlobPrefs->GetMainProcessPriority());
	
	m_pDump->LoadStrings();

	EMULE_CATCH

	CEmuleDlg		dlg;

	m_pdlgEmule = &dlg;
	m_pMainWnd = &dlg;

	EMULE_TRY

	// DonGato
	switch (m_pGlobPrefs->GetHashingPriority())
	{
		case 0:
			m_fileHashControl.SetProcessingPriority(ePIdle);
			break;
		case 1:
			m_fileHashControl.SetProcessingPriority(ePLower);
			break;
		default:
			m_fileHashControl.SetProcessingPriority(ePStandard);
			break;
	}

	InitURLs(); // DonGato: ICC customization

//	Change extension for help file
	CString		strHelpFile = m_pszHelpFilePath;

	if (m_pGlobPrefs->GetLanguageID()==MAKELANGID(LANG_GERMAN,SUBLANG_DEFAULT))
	{
		int		pos = strHelpFile.ReverseFind('.');

		if (pos > 0)
			strHelpFile = strHelpFile.Left(pos) + _T(".de.chm");
	}
	strHelpFile.Replace(_T(".HLP"), _T(".chm"));
	free((void*)m_pszHelpFilePath);
	m_pszHelpFilePath = _tcsdup(strHelpFile);

	m_fileHashControl.Init(); // Lord KiRon

// Barry - Auto-take ed2k links
	if (m_pGlobPrefs->AutoTakeED2KLinks())
		Ask4RegFix(true);

	try
	{
		CString		sDbHome = CString(m_pGlobPrefs->GetAppDir()) + _T("Db");

		::CreateDirectory(sDbHome, NULL);	// In case it doesn't exist

		pDbEnv = new DbEnv(0);
		pDbEnv->set_cachesize(0, 256 * 1024, 0);	// 256kb cache size
		USES_CONVERSION;
		pDbEnv->open( CT2CA(sDbHome), DB_CREATE | DB_INIT_LOCK | DB_INIT_LOG
									| DB_INIT_MPOOL | DB_INIT_TXN | DB_RECOVER
									| DB_THREAD, 0 );
		pDbSLS = new Db(pDbEnv, 0);
		pDbSLS->set_pagesize(16 * 1024); // 16Kb page size
		pDbSLS->open(NULL, "Main.db", "SLS", DB_BTREE, DB_CREATE | DB_THREAD, 0);
#if JUMPSTART
        CJumpstarter::OpenDatabases(pDbEnv);
#endif // JUMPSTART
#if CRUISECONTROL
        CCruiseControl::OpenDatabases(pDbEnv);
#endif // CRUISECONTROL
	}
    catch (DbRunRecoveryException &pDbEnv)
	{
        AfxMessageBox(CString(pDbEnv.what()));
		return FALSE;
    }
	catch (DbException &pDbEnv)
	{
		pDbSLS = NULL;
		pDbEnv = NULL;
		TRACE1("Problems create database objects: %s. Closing.\n", pDbEnv.what());
		return FALSE;
	}
 	catch(...)
	{
		pDbSLS = NULL;
		pDbEnv = NULL;
		TRACE("Problems create database objects. Closing.\n");
		return FALSE;
	}

	m_pDownloadList = new CDownloadList();
    m_pClientList = new CClientList();
//	Construct the Friends List from "emfriends.met"
	m_pFriendList = new CFriendList();
	m_pSearchList = new CSearchList();
//	Construct the Known Files list from "known.met"
	m_pKnownFilesList = new CKnownFileList(m_pGlobPrefs->GetConfigDir());
//	Construct an _empty_ Server list.
	m_pServerList = new CServerList(m_pGlobPrefs);
#ifdef OLD_SOCKETS_ENABLED
	m_pServerConnect = new CServerConnect(m_pServerList,g_eMuleApp.m_pGlobPrefs); // in "sockets.cpp" for some reason
	m_pSharedFilesList = new CSharedFileList(m_pGlobPrefs,m_pServerConnect,m_pKnownFilesList);
	m_pListenSocket = new CListenSocket(m_pGlobPrefs);
	m_pClientUDPSocket	= new CClientUDPSocket();
#else
	m_pSharedFilesList = new CSharedFileList(m_pGlobPrefs,NULL,m_pKnownFilesList);
#endif //OLD_SOCKETS_ENABLED
	m_pClientCreditList = new CClientCreditsList(m_pGlobPrefs);
	m_pDownloadQueue = new CDownloadQueue(m_pGlobPrefs,m_pSharedFilesList);	// bugfix - do this before creating the m_pUploadQueue
	m_pUploadQueue = new CUploadQueue(m_pGlobPrefs);
	m_pIPFilter 	= new CIPFilter();
	m_pWebServer = new CWebServer();
#ifdef OLD_SOCKETS_ENABLED
#ifdef MOBILE_MULE
	m_pMMServer = new CMMServer();
#endif //MOBILE_MULE
#endif //OLD_SOCKETS_ENABLED
	m_pSMTPConnection = new CSMTPConnection();
	m_pFakeCheck 	= new CFakecheck(); //FakeCheck


	// reset statistic values
	g_eMuleApp.stat_sessionReceivedBytes=0;
	g_eMuleApp.stat_sessionSentBytes=0;
	g_eMuleApp.stat_reconnects=0;
	g_eMuleApp.stat_transferStarttime=0;
	g_eMuleApp.stat_serverConnectTime=0;
	g_eMuleApp.stat_filteredclients=0;

	EMULE_CATCH

	INT_PTR nResponse = dlg.DoModal();

#ifdef _DEBUG
	newMemState.Checkpoint();
	if (diffMemState.Difference(oldMemState, newMemState))
	{
		TRACE("Memory usage:\n");
		diffMemState.DumpStatistics();
	}
#endif //_DEBUG

	return FALSE;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CEmuleApp::ProcessCommandline()
{
	HWND maininst = NULL;
	bool bAlreadyRunning = false;

	CCommandLineInfo		cmdInfo;
	ParseCommandLine(cmdInfo);
	
//	TODO: I'm almost certain there's an easier way to do this. In older Windows programs,
//		the runtime would pass the WinMain the instance handle of an existing instance
//		of the application.
	m_hMutexOneInstance = CreateMutex(NULL, FALSE,_T(EMULE_GUID));

	bAlreadyRunning = ( ::GetLastError() == ERROR_ALREADY_EXISTS ||::GetLastError() == ERROR_ACCESS_DENIED);
   	if (bAlreadyRunning) EnumWindows(SearchEmuleWindow, (LPARAM)&maininst);

//	If the shell commmand was to open the program (as opposed to print, etc.)...
	if (cmdInfo.m_nShellCommand == CCommandLineInfo::FileOpen)
	{
		CString command = cmdInfo.m_strFileName;

	//	If the file name to open is in the form of an URL...
		if (command.Find(_T("://"))>0)
		{
		//
		//	Just forward the URL to the existing eMule instance
			m_sendStruct.cbData = command.GetLength()+1;
			m_sendStruct.dwData = OP_ED2KLINK;
			m_sendStruct.lpData = command.GetBuffer();
 		   	if (maininst)
			{
      			SendMessage(maininst,WM_COPYDATA,(WPARAM)0,(LPARAM) (PCOPYDATASTRUCT) &m_sendStruct);
     	 		return 0;
			} 
    		else 
      			m_pstrPendingLink = new CString(command);
		}
		else
		{
		//
		//	Forward the command line to the existing eMule instance
			m_sendStruct.cbData = command.GetLength()+1;
			m_sendStruct.dwData = OP_CLCOMMAND;
			m_sendStruct.lpData = command.GetBuffer();
    		if (maininst)
    		{
      			SendMessage(maininst,WM_COPYDATA,(WPARAM)0,(LPARAM) (PCOPYDATASTRUCT) &m_sendStruct);
      			return 0;
			}
		}
    }
 	if(maininst || bAlreadyRunning)
		return 1;
	else
		return 2;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	SearchEmuleWindow() is an EnumWindows callback proc. It sends an eMule Windows user message to the
//		window 'hWnd' and waits for a response.
BOOL CALLBACK CEmuleApp::SearchEmuleWindow(HWND hWnd, LPARAM lParam)
{
	DWORD dwMsgResult;
	LRESULT res = ::SendMessageTimeout(hWnd,UWM_ARE_YOU_EMULE,0, 0,SMTO_BLOCK |SMTO_ABORTIFHUNG,10000,&dwMsgResult);

//	If the window didn't respond, tell EnumWindows to continue with the next window
	if(res == 0)
		return TRUE;

//	If we got a response...
	if (dwMsgResult == UWM_ARE_YOU_EMULE)
	{
	//	Return the window handle in '*lParam'.
		HWND * target = (HWND *)lParam;

		*target = hWnd;
		return FALSE;
	} 
	return TRUE;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CEmuleApp::UpdateReceivedBytes(int32 bytesToAdd) {
	SetTimeOnTransfer();
	stat_sessionReceivedBytes+=bytesToAdd;
}

void CEmuleApp::UpdateSentBytes(int32 bytesToAdd) {
	SetTimeOnTransfer();
	stat_sessionSentBytes+=bytesToAdd;
}

void CEmuleApp::SetTimeOnTransfer() {
	if (stat_transferStarttime>0) return;
	
	stat_transferStarttime=GetTickCount();
}

CString CEmuleApp::StripInvalidFilenameChars(CString strText, bool bKeepSpaces)
{
	LPTSTR pszBuffer = strText.GetBuffer();
	LPTSTR pszSource = pszBuffer;
	LPTSTR pszDest = pszBuffer;

	while (*pszSource != '\0')
	{
		if (!((*pszSource <= 31 && *pszSource >= 0)	|| // lots of invalid chars for filenames in windows :=)
			*pszSource == '\"' || *pszSource == '*' || *pszSource == '<'  || *pszSource == '>' ||
			*pszSource == '?'  || *pszSource == '|' || *pszSource == '\\' || *pszSource == '/' || 
			*pszSource == ':') )
		{
			if (!bKeepSpaces && *pszSource == ' ')
				*pszDest = '.';
			*pszDest = *pszSource;
			pszDest++;
		}
		pszSource++;
	}
	*pszDest = '\0';
	strText.ReleaseBuffer();
	return strText;
}

bool CEmuleApp::CopyTextToClipboard(const CString& strText) const
{
	//allocate global memory & lock it
	HGLOBAL hGlobal = GlobalAlloc(GHND|GMEM_SHARE,strText.GetLength() + 1);
	if(hGlobal == NULL)
		return false;

	LPTSTR pGlobal = static_cast<LPTSTR>(GlobalLock(hGlobal));
	if( pGlobal == NULL )
		return false;

	//copy the text
	_tcscpy(pGlobal,strText);

	GlobalUnlock(hGlobal);

	//Open the Clipboard and insert the handle into the global memory
	if( OpenClipboard(NULL) == NULL )
		return false;

	if(!EmptyClipboard())
		return false;

	if( SetClipboardData(CF_TEXT,hGlobal) == NULL )
		return false;

	CloseClipboard();

	return true;
}

// START - enkeyDEV(Ottavio84) -Download all ed2ks in clipboard-
CString CEmuleApp::CopyTextFromClipboard() 
{
	HGLOBAL	hglb;
	LPTSTR  lptstr;
	CString	retstring;
	if (!IsClipboardFormatAvailable(CF_TEXT)) 
		return retstring;
	if (!OpenClipboard(NULL)) 
		return retstring;

	hglb = GetClipboardData(CF_TEXT);
	if (hglb != NULL) 
	{ 
		lptstr = (LPTSTR)GlobalLock(hglb);
		if (lptstr != NULL) retstring = lptstr;			
	} 
	CloseClipboard();
	return retstring;
}
// END - enkeyDEV(Ottavio84) -Download all ed2ks in clipboard-

void CEmuleApp::OnlineSig() // Added By Bouc7 
{ 
	if (!g_eMuleApp.m_pGlobPrefs->IsOnlineSignatureEnabled()) return;

    CString fullpath;
    fullpath.Format(_T("%sonlinesig.dat"),m_pGlobPrefs->GetConfigDir());
    CFile file;
    if (!file.Open(fullpath,CFile::modeCreate|CFile::modeReadWrite)){ 
		AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERROR_SAVEFILE)+CString(" OnlineSig.dat"));	    
		return;
    } 
    CString buffer;
#ifdef OLD_SOCKETS_ENABLED
    if (m_pServerConnect->IsConnected()) 
    { 
      file.Write("1",1);
      file.Write("|",1);
      file.Write(m_pServerConnect->GetCurrentServer()->GetListName(),m_pServerConnect->GetCurrentServer()->GetListName().GetLength());
      // Not : file.Write(m_pServerConnect->GetCurrentServer()->GetListName(),strlen(m_pServerConnect->GetCurrentServer()- >GetRealName()));

      file.Write("|",1);
      file.Write(m_pServerConnect->GetCurrentServer()->GetFullIP(),m_pServerConnect->GetCurrentServer()->GetFullIP().GetLength());
      file.Write("|",1);
	  buffer.Format(_T("%d"), m_pServerConnect->GetCurrentServer()->GetPort());
      file.Write(buffer,buffer.GetLength());
    } 
    else 
#endif //OLD_SOCKETS_ENABLED
      file.Write("0",1);

    file.Write("\n",1);
    buffer.Format(_T("%.1f"),static_cast<double>(m_pDownloadQueue->GetDataRate())/1024.0);
    file.Write(buffer,buffer.GetLength());
    file.Write("|",1);
    buffer.Format(_T("%.1f"),static_cast<double>(m_pUploadQueue->GetDataRate())/1024.0);
    file.Write(buffer,buffer.GetLength());
    file.Write("|",1);
    buffer.Format(_T("%d"), m_pUploadQueue->GetWaitingUserCount());
    file.Write(buffer,buffer.GetLength());

    file.Close();
} //End Added By Bouc7
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// [FoRcHa] 
int CEmuleApp::ExitInstance()
{
	EMULE_TRY
	m_pdlgEmule = NULL;	//[TwoBottle Mod]

	m_fileHashControl.Destroy(); // Lord KiRon
	
	//Cax2 changed to avoid crash on exit/corrupted .met data
#ifdef OLD_SOCKETS_ENABLED
	delete m_pListenSocket;
	m_pListenSocket = NULL;
	delete m_pClientUDPSocket;
	m_pClientUDPSocket = NULL;
	delete m_pServerConnect;
	m_pServerConnect = NULL;
#endif //OLD_SOCKETS_ENABLED
	delete m_pServerList;
	m_pServerList = NULL;
	delete m_pKnownFilesList;
	m_pKnownFilesList = NULL;
	delete m_pDownloadQueue;	//Cax2 - this calls the save part.met functions
	m_pDownloadQueue = NULL;
	delete m_pUploadQueue;
	m_pUploadQueue = NULL;
	delete m_pSharedFilesList;
	m_pSharedFilesList = NULL;	
	delete m_pSearchList;
	m_pSearchList = NULL;
	delete m_pClientCreditList;
	m_pClientCreditList = NULL;
	delete m_pClientList;
	m_pClientList = NULL;
	delete m_pDownloadList;
	m_pDownloadList = NULL;
	delete m_pFriendList;
	m_pFriendList = NULL;
	delete m_pGlobPrefs;
	m_pGlobPrefs = NULL;
	delete m_pIPFilter;
	m_pIPFilter = NULL;
	delete m_pWebServer; // kuchin
	m_pWebServer = NULL;
#ifdef OLD_SOCKETS_ENABLED
#ifdef MOBILE_MULE
	delete m_pMMServer;
	m_pMMServer = NULL;
#endif //MOBILE_MULE
#endif //OLD_SOCKETS_ENABLED
	delete m_pSMTPConnection;
	m_pSMTPConnection = NULL;

	try
	{
#if JUMPSTART
        CJumpstarter::CloseDatabases();
#endif // JUMPSTART
#if CRUISECONTROL
        CCruiseControl::CloseDatabases();
#endif // CRUISECONTROL
		pDbSLS->close(0);
		pDbEnv->txn_checkpoint(0,0,0);
		pDbEnv->close(0);
		delete pDbSLS;
		delete pDbEnv;
	}
	catch(DbException &pDbEnv)
	{
		TRACE1("Problems delete database objects: %s.\n", pDbEnv.what());
	}
	catch(...)
	{
		TRACE("Problems delete database objects.\n");
	}

	m_app_state = APP_STATE_DONE;	

	if(m_pDump) {
		delete m_pDump;
		m_pDump = NULL;
	}

	CloseHandle(m_hMutexOneInstance); // FoRcHa

	return CWinApp::ExitInstance();
	EMULE_CATCH2
	return 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	OnHelp() is the handler for the Windows Help message
void CEmuleApp::OnHelp()
{
//	Lets just open the helpfile by associated program, instead of more windows-dependency :)
/*	CFileFind	ff;

	if (ff.FindFile(m_pszHelpFilePath, 0))
		ShellOpenFile(m_pszHelpFilePath);
	else
		AfxMessageBox(GetResString(IDS_ERR_NOHELP), MB_OK | MB_ICONERROR);

	ff.Close();*/
	ShellExecute(NULL, "open", "http://emuleplus.info/forum/index.php?showforum=23", NULL, NULL, SW_SHOW);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CEmuleApp::GetFileTypeSystemImageIdx(LPCTSTR pszFilePath, int iLength /* = -1 */)
{
//	TODO: This has to be MBCS aware..
	DWORD dwFileAttributes;
	LPCTSTR pszCacheExt = NULL;
	if (iLength == -1)
		iLength = _tcslen(pszFilePath);
	if (iLength > 0 && (pszFilePath[iLength - 1] == _T('\\') || pszFilePath[iLength - 1] == _T('/'))){
		// it's a directory
		pszCacheExt = _T("\\");
		dwFileAttributes = FILE_ATTRIBUTE_DIRECTORY;
	}
	else{
		dwFileAttributes = FILE_ATTRIBUTE_NORMAL;
		// search last '.' character *after* the last '\\' character
		for (int i = iLength - 1; i >= 0; i--){
			if (pszFilePath[i] == _T('\\') || pszFilePath[i] == _T('/'))
				break;
			if (pszFilePath[i] == _T('.')) {
				// point to 1st character of extension (skip the '.')
				pszCacheExt = &pszFilePath[i+1];
				break;
			}
		}
		if (pszCacheExt == NULL)
			pszCacheExt = _T("");	// empty extension
	}

	// Search extension in "ext->idx" cache.
	LPVOID vData;
	if (!m_aExtToSysImgIdx.Lookup(pszCacheExt, vData)){
		// Get index for the system's small icon image list
		SHFILEINFO sfi;
		DWORD dwResult = SHGetFileInfo(pszFilePath, dwFileAttributes, &sfi, sizeof(sfi),
									   SHGFI_USEFILEATTRIBUTES | SHGFI_SYSICONINDEX | SHGFI_SMALLICON);
		if (dwResult == 0)
			return 0;
		ASSERT( m_hSystemImageList == NULL || m_hSystemImageList == (HIMAGELIST)dwResult );
		m_hSystemImageList = (HIMAGELIST)dwResult;

		// Store icon index in local cache
		m_aExtToSysImgIdx.SetAt(pszCacheExt, (LPVOID)sfi.iIcon);
		return sfi.iIcon;
	}
	// Return already cached value
	// Elandal: Assumes sizeof(void*) == sizeof(int)
	return (int)vData;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	InitURLs() parses the domains for the Internet Callback Check out of the pref string into
//		the string array 'm_ICCURLs'.
void CEmuleApp::InitURLs()
{
/*	CString		URLs = m_pGlobPrefs->GetURLsForICC();
	
	m_ICCURLs.RemoveAll();

	int		n = 0;
	int		m;

	for (m = URLs.Find(_T('|'), n); m >= 0; m = URLs.Find(_T('|'), n))
	{ 
		if (m > n + 1)
			m_ICCURLs.Add(URLs.Mid(n,m-n));
		n = m + 1;
	}
*/
	CString strList(_T(""));	//list of pipe-separated tokens
	CString resToken(_T(""));	//one token out of strList
	int curPos = 0;				//position for tokenize() (by reference!)

	strList.SetString(g_eMuleApp.m_pGlobPrefs->GetURLsForICC().MakeLower());
	curPos = 0;
	resToken.SetString(strList.Tokenize(_T("|"),curPos));
	while (!resToken.IsEmpty())
	{
		m_ICCURLs.Add(resToken);
		resToken.SetString(strList.Tokenize(_T("|"),curPos));
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CEmuleApp::PreTranslateMessage(MSG* pMsg)
{
	if (pMsg->message == WM_KEYDOWN || pMsg->message == WM_SYSKEYDOWN)
	{
		int iMessage = 0;
		short nCode = GetCodeFromPressedKeys(pMsg);

		if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalMinimize))
		{
			m_pdlgEmule->TrayShow();
			m_pdlgEmule->ShowWindow(SW_HIDE);
			return TRUE;
		}
		else if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalSwitchWindow))
		{
			if (m_pdlgEmule->m_pdlgActive == &m_pdlgEmule->m_wndServer)
				iMessage = IDC_TOOLBARBUTTON + MTB_TRANSFER;
			else if (m_pdlgEmule->m_pdlgActive == &m_pdlgEmule->m_wndTransfer)
				iMessage = IDC_TOOLBARBUTTON + MTB_SEARCH;
			else if (m_pdlgEmule->m_pdlgActive == &m_pdlgEmule->m_dlgSearch)
				iMessage = IDC_TOOLBARBUTTON + MTB_SHAREDFILES;
			else if (m_pdlgEmule->m_pdlgActive == &m_pdlgEmule->m_wndSharedFiles)
				iMessage = IDC_TOOLBARBUTTON + MTB_MESSAGES;
			else if (m_pdlgEmule->m_pdlgActive == &m_pdlgEmule->m_wndChat)
				iMessage = IDC_TOOLBARBUTTON + MTB_IRC;
			else if (m_pdlgEmule->m_pdlgActive == &m_pdlgEmule->m_wndIRC)
				iMessage = IDC_TOOLBARBUTTON + MTB_STATISTICS;
			else if (m_pdlgEmule->m_pdlgActive == &m_pdlgEmule->m_dlgStatistics)
				iMessage = IDC_TOOLBARBUTTON + MTB_SERVERS;
		}
		else if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalShowServer))
			iMessage = IDC_TOOLBARBUTTON + MTB_SERVERS;
		else if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalShowTransfer))
			iMessage = IDC_TOOLBARBUTTON + MTB_TRANSFER;
		else if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalShowSearch))
			iMessage = IDC_TOOLBARBUTTON + MTB_SEARCH;
		else if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalShowShare))
			iMessage = IDC_TOOLBARBUTTON + MTB_SHAREDFILES;
		else if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalShowChat))
			iMessage = IDC_TOOLBARBUTTON + MTB_MESSAGES;
		else if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalShowIRC))
			iMessage = IDC_TOOLBARBUTTON + MTB_IRC;
		else if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalShowStats))
			iMessage = IDC_TOOLBARBUTTON + MTB_STATISTICS;
		else if (nCode == m_pGlobPrefs->GetShortcutCode(IndexGalShowPrefs))
			iMessage = IDC_TOOLBARBUTTON + MTB_PREFS;

		if (iMessage > 0)
		{
			m_pdlgEmule->PostMessage(WM_COMMAND, iMessage);
			return TRUE;
		}
	}

	return __super::PreTranslateMessage(pMsg);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

