//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "stdafx.h"
#include "server.h"
#include "opcodes.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CServer::CServer(ServerMet_Struct *pServerMet)
{	
	m_uPort = pServerMet->m_uPort;
	m_dwTagCount = 0;
	m_dwIP = pServerMet->m_dwIP;

	in_addr		host;

	host.S_un.S_addr = m_dwIP;
	m_strFullIP = inet_ntoa(host);
	m_dwFiles = 0;
	m_dwNumUsers = 0;
	m_ePreferences = SERVERPRIORITY_LOW;
	m_dwPingTime = 0;
	m_dwFailedCount = 0; 
	lastpinged = 0;
	lastpingedtime = 0;
	lastdescpingedcount = 0;
	m_bIsStaticServerMember = false;
	m_dwMaxUsers=0;
	m_dwSoftMaxFiles = 0;
	m_dwHardMaxFiles = 0;
	m_dwTCPFlags = 0;
	m_dwUDPFlags = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CServer::CServer(uint16 in_port, const CString& i_addr){	
	m_uPort = in_port;
	m_dwTagCount = 0;
	if (inet_addr(i_addr) == INADDR_NONE){
		m_strDynIP = i_addr;
		m_dwIP = 0;
	}
	else{
		m_dwIP = inet_addr(i_addr);
		m_strDynIP = "";
	}
	in_addr host;
	host.S_un.S_addr = m_dwIP;
	m_strFullIP = inet_ntoa(host);
	m_dwFiles = 0;
	m_dwNumUsers = 0;
	m_ePreferences = SERVERPRIORITY_LOW;
	m_dwPingTime = 0;	
	m_dwFailedCount = 0; 
	lastpinged = 0;
	lastpingedtime = 0;
	lastdescpingedcount = 0;
	m_bIsStaticServerMember = false;
	m_dwMaxUsers=0;
	m_dwSoftMaxFiles = 0;
	m_dwHardMaxFiles = 0;
	m_dwTCPFlags = 0;
	m_dwUDPFlags = 0;
}

// copy constructor
CServer::CServer(const CServer& pOld)
{	
	for(POSITION pos = pOld.m_tagList.GetHeadPosition(); pos != NULL; pOld.m_tagList.GetNext(pos))
	{
		CServerTag* pOldTag = pOld.m_tagList.GetAt(pos);
		m_tagList.AddTail(new CServerTag(*pOldTag));
	}
	m_uPort = pOld.m_uPort;
	m_dwIP = pOld.m_dwIP; 
	m_bIsStaticServerMember = pOld.IsStaticMember();
	m_dwTagCount = pOld.m_dwTagCount;
	m_strFullIP = pOld.m_strFullIP;
	m_dwFiles = pOld.m_dwFiles;
	m_dwNumUsers = pOld.m_dwNumUsers;
	m_ePreferences = pOld.m_ePreferences;
	m_dwPingTime = pOld.m_dwPingTime;
	m_dwFailedCount = pOld.m_dwFailedCount; 
	lastpinged = pOld.lastpinged;
	lastpingedtime = pOld.lastpinged;
	lastdescpingedcount = pOld.lastdescpingedcount;
	m_dwMaxUsers = pOld.m_dwMaxUsers;	
	description = pOld.description;	
	m_strListName = pOld.m_strListName;
	m_strDynIP = pOld.m_strDynIP;	
	m_dwSoftMaxFiles = pOld.m_dwSoftMaxFiles;
	m_dwHardMaxFiles = pOld.m_dwHardMaxFiles;
	m_strVersion = pOld.m_strVersion;
	m_dwTCPFlags = pOld.m_dwTCPFlags;
	m_dwUDPFlags = pOld.m_dwUDPFlags;
}

CServer::~CServer(){		
	for(POSITION pos = m_tagList.GetHeadPosition(); pos != NULL; m_tagList.GetNext(pos))
		delete m_tagList.GetAt(pos);
	m_tagList.RemoveAll();	
}

bool CServer::AddTagFromFile(CSafeFile& servermet)
{
	CServerTag* tag = new CServerTag(servermet);
	switch(tag->GetSpecialTag())
	{		
		case ST_SERVERNAME:						
			m_strListName = tag->GetStringValue();						
			delete tag;
			break;
		case ST_DESCRIPTION:						
			description = tag->GetStringValue();						
			delete tag;
			break;
		case ST_PREFERENCE:
			// Modified by Tarod [thx Hein] - import server preferences and translate it
			m_ePreferences = (EnumServerPriority)ed2k2eMule(tag->GetIntValue());
			delete tag;
			break;
		case ST_PING:
			m_dwPingTime = tag->GetIntValue();
			delete tag;
			break;
		case ST_DYNIP:						
			m_strDynIP = tag->GetStringValue();						
			delete tag;
			break;
		case ST_FAIL:
			m_dwFailedCount = tag->GetIntValue();
			delete tag;
			break;
		case ST_LASTPING:
			lastpingedtime = tag->GetIntValue();
			delete tag;
			break;
		case ST_MAXUSERS:
			m_dwMaxUsers = tag->GetIntValue();
			delete tag;
			break;
		case ST_SOFTFILES:
			m_dwSoftMaxFiles = tag->GetIntValue();
			delete tag;
			break;
		case ST_HARDFILES:
			m_dwHardMaxFiles = tag->GetIntValue();
			delete tag;
			break;
		case ST_VERSION:
			if (tag->GetType() == TAGTYPE_STRING)
			m_strVersion = tag->GetStringValue();
			delete tag;
			break;
		case ST_UDPFLAGS:
			if (tag->GetType() == TAGTYPE_INT)
			m_dwUDPFlags  =  tag->GetIntValue();
			delete tag;
			break;
		default:
			if (tag->GetSpecialTag())
			{
				tag->SetTagName(_T("Unknown"));
				AddTag(tag);
			}
			else if (tag->GetTagName() == _T("files"))
			{
				m_dwFiles = tag->GetIntValue();
				delete tag;
			}
			else if (tag->GetTagName() == _T("users"))
			{
				m_dwNumUsers = tag->GetIntValue();
				delete tag;
			}
			else
				AddTag(tag);
	}
	return true;
}

void CServer::FillWindowTags(CTreeCtrl* wnd,HTREEITEM rootitem){
	POSITION pos;
	CString buffer;
	if (!description.IsEmpty()) {
		buffer.Format(GetResString(IDS_DESC),description);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	buffer.Format(GetResString(IDS_IP)+_T(": %s"),m_strFullIP);
	wnd->InsertItem(buffer,-1,-1,rootitem);
	buffer.Format(GetResString(IDS_PORT)+_T(": %i"),m_uPort);
	wnd->InsertItem(buffer,-1,-1,rootitem);
	if (m_dwPingTime) {
		buffer.Format(GetResString(IDS_PING)+_T(": %i"),m_dwPingTime);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	if (m_dwNumUsers) {
		buffer.Format(GetResString(IDS_USERS),m_dwNumUsers);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	if (m_dwFiles) {
		buffer.Format(GetResString(IDS_FILES)+_T(": %i"),m_dwFiles);
		wnd->InsertItem(buffer,-1,-1,rootitem);
	}
	CServerTag* cur_tag;
	for(int i = 0; i != m_tagList.GetCount();i++) {
		pos = m_tagList.FindIndex(i);
		cur_tag = m_tagList.GetAt(pos);
		if (cur_tag->GetSpecialTag())
			continue;
		if (cur_tag->GetType() == 2)
			buffer.Format(_T("%s: %s"),cur_tag->GetTagName(),cur_tag->GetStringValue());
		else if (cur_tag->GetType() == 3)
			buffer.Format(_T("%s: %i"),cur_tag->GetTagName(),cur_tag->GetIntValue());
		else
			continue;
		wnd->InsertItem(buffer,-1,-1,rootitem);
   }
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CString CServer::GetAddress() const
{
	if (!m_strDynIP.IsEmpty())
		return m_strDynIP;
	else
		return m_strFullIP;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	SetID() sets the IP address of the Server to 'iNewIP'.
void CServer::SetID(uint32 iNewIP)
{
	m_dwIP = iNewIP;

//
//	Stringify the numerical IP using "." notation
	in_addr		host;

	host.S_un.S_addr = m_dwIP;
	m_strFullIP = inet_ntoa(host);
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServer::HasSameAddress(const CServer& server) const {

	if ((m_dwIP == server.m_dwIP) && ((m_uPort == server.m_uPort) || m_uPort == 0 || server.m_uPort == 0))
	{
		if (m_dwIP != 0)
			return true;
		else if (m_strDynIP == server.m_strDynIP)
			return true;
	}
	return false;
}

/**
 * convert eDonkey pref values to eMule pref values
 * The correct server preferences (as used by eDonkey) are: Low=2, Normal=0, High=1 
 * @param srvPref preference as found in server.met
 * @return preference as used in eMule
 */
uint32 CServer::ed2k2eMule(uint32 srvPref)
{
	uint32 mPref;
	switch (srvPref)
	{
	case 0:
		mPref = PR_NORMAL;
		break;
	case 1:
		mPref = PR_HIGH;
		break;
	case 2:
		mPref = PR_LOW;
		break;
	default:
		mPref = PR_LOW;
		break;
	}

	return mPref;
}

/**
 * convert eMule pref values to eDonkey pref values
 * The correct server preferences (as used by eDonkey) are: Low=2, Normal=0, High=1 
 * @param srvPref preference as found in eMule
 * @return preference as used in server.met
 */
uint32 CServer::eMule2ed2k(uint32 srvPref)
{
	uint32 dPref;
	switch (srvPref)
	{
	case PR_NORMAL:
		dPref = 0;
		break;
	case PR_HIGH:
		dPref = 1;
		break;
	case PR_LOW:
		dPref = 2;
		break;
	default:
		dPref = 2;
		break;
	}

	return dPref;
}

bool CServer::HasPublicAddress() const
{
	in_addr host;
	host.S_un.S_addr = m_dwIP;

	int b1 = host.S_un.S_un_b.s_b1;
	int b2 = host.S_un.S_un_b.s_b2;

	if (b1 == 172 && (b2 >= 16 && b2 <= 31))
		return false;

	if (b1 == 192 && b2 == 168)
		return false;

	if (b1 == 0 || b1 == 10 || b1 == 127 || GetPort() == 0)
		return false;

	return true;
}
